# -*- encoding: UTF-8 -*-

'''
Created on Mar 3, 2010

@author: maemo
'''

import  webbrowser;
import  pickle;
from zourite.core import  zourite, plugin
import  datetime
import  gtk

from zourite.common import version


version.getInstance().submitRevision("$Revision: 105 $")

class Zourite(zourite.Zourite):
    
    mystatus = ""
    
    def __init__(self):
        self.mystatus = "my status"
        self.settings = zourite.Settings()
    
    def get_plugin_by_id(self, networkid):
        return self;
    
    def get_plugin_id(self):
        return "mock_plugin"
    
    def getAvailablePlugins(self):
        return [self]
    
    def getShortProfilesForNetUpdateConnection(self, netUpdateConn):
        p1 = plugin.ShortProfile();
        p1.firstname = netUpdateConn.connection.firstname
        p1.lastname = netUpdateConn.connection.lastname
        p1.headline = netUpdateConn.connection.headline

        p2 = plugin.ShortProfile();
        p2.firstname = netUpdateConn.newConnection.firstname
        p2.lastname = netUpdateConn.newConnection.lastname
        p2.headline = netUpdateConn.newConnection.headline

        
        return p1,p2
    
    def getGtkImageForProfile(self, shortProfile):
         return plugin.ImageProxyNone().getGtkImage()
    
    def getGtkPixBufForPerson(self, persone):
        return plugin.ImageProxyNone().getGdkPixbuf()
    
    def getPluginLogoPixbuf(self):
          return gtk.gdk.pixbuf_new_from_file("linkedin.png")
    
    def getGtkPixBufForProfile(self, sp):
         return plugin.ImageProxyNone().getGdkPixbuf()
    
    def getFullProfile(self, contactId):
        fullProfile = plugin.FullProfile()
        fullProfile.id=contactId
        fullProfile.firstname = "firstane"
        fullProfile.lastname  ="lastname"
        fullProfile.networkId = "linkedin"
        fullProfile.headline = "headline"
        
        fullProfile.summary = "I've spent a lof of time making program  mainly in financial concern " \
        + "but now i work in a open source company and i dare joining an innovative project dealing " \
        + "with difficul subject"
        
        fullProfile.industry="Software"
        
        p = plugin.Position("Software engineer","Company",datetime.date(2000,1,1));
        p.summary = "work on an CRM project"
        
    
        fullProfile.positions = [p]
        
        return fullProfile
        
    def sendInvitationRequest(self, contactId, subject, body):
        pass;
    
    def sendMessageToProfile(self, shortProfile, subject, body):
        pass
    
    def sendMessageToNetwork(self, body):
        pass
    
    def getMyConnections(self):
        allMyConnection = []
        for i in range(10):
            p = plugin.ShortProfile()
    
            p.firstname = "firstname" + str(i)
            p.headline = "headline" + str(i)
            p.id = "ABCD" + str(i)
            p.lastname = "lastname" + str(i)
            p.phone = "0123456789"
            p.email = "zourite@bressure.net"
    
            allMyConnection.append(p)
            
        return allMyConnection
    
    
    def getShortProfile(self,contactId):
        p1 = plugin.ShortProfile();
        p1.firstname = "Jean"
        p1.lastname = "Dupon"
        p1.headline = "Account manager " + str(contactId)

               
        return p1
        
    def getMyStatus(self):
        return self.mystatus
     
    def setMyStatus(self, status):
       self.mystatus = status
       
    def clearMyStatus(self):
        self.mystatus = ""
        
    def getAllMyStatus(self):
        '''
        Return a set of all status grabbed from every network plugin.
        '''
        status = []
        if len(self.mystatus) > 0:
             status.append(self.mystatus)
        return set(status)
        

    def getNetworkUpdate(self): 
        allMyNetworkUpdate = []
        
        for i in range(10):        
            
            
            p1 = plugin.Person()
            p1.firstname = "firstname" + str(i)
            p1.headline = "headline" + str(i)
            p1.id = "ABCD" + str(i)
            p1.lastname = "lastname" + str(i)
           
            
            p2 = plugin.Person()
            p2.firstname = "firstname" + str(i)
            p2.headline = "headline" + str(i)
            p2.id = "ABCD" + str(i)
            p2.lastname = "lastname" + str(i)
           

            netupdate = plugin.NetworkUpdateConnection("12345678", p1,p2,"linkedin")
            allMyNetworkUpdate.append(netupdate)
        
        return allMyNetworkUpdate


# -*- encoding: UTF-8 -*-
'''
Created on 28 févr. 2010

@author: thierry
'''

import gtk
import hildon
from zourite.core import zourite, plugin # TODO i would like to remove this
import logging
import sys, traceback
import threading
import webbrowser
import mock

import dbus

import gdata.projecthosting.client
import gdata.projecthosting.data
import gdata.gauth
import gdata.client
import gdata.data
import atom.http_core
import atom.core

from zourite.common import version


version.getInstance().submitRevision("$Revision: 155 $")


class ZouriteGui():
   
    program = None 
    zcore = None
    
    def __init__(self, mocked=False):
   
       #zcore = zourite.Zourite()
       self.program = hildon.Program.get_instance()
       if mocked :
           self.zcore = mock.Zourite()
       else:
           self.zcore = zourite.Zourite()
       
       self.init_main_view()   

    def _restore_default_settings(self):
        self.zcore.applyDefaultSettings()
        self.show_banner_information("default settings restored")
        # remove  the view
        hildon.WindowStack.get_default().pop_1()


    def _save_settings(self, settings):
        self.zcore.applyAndSaveSettings(settings)
        self.show_banner_information("setting saved")
        # remove  the view
        hildon.WindowStack.get_default().pop_1()


    def _refresh_from_menu(self):
        '''
    the user has request to refresh the view
    '''
        view = hildon.WindowStack.get_default().peek()        
        # disable the cache mechanisme of zourite
        self.zcore.setForceRefresh(True)
        try:
            view.refresh()
        finally:
            self.zcore.setForceRefresh(False)


    def _show_contact_list(self):
        
        window = ContactListView(self.select_contact, self.place_call, self.place_email, self.zcore)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()

    def _show_network_update_connection_detail(self, netUpdateConn):
       
        myconn, newconn = self.zcore.getShortProfilesForNetUpdateConnection(netUpdateConn)
        window = NetworkUpdateConnectionView(myconn, newconn, self.show_detail_profil, self.show_invite_profile, self.zcore)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
        
    def _show_network_update_self_connection_detail(self, netUpdateConn):
       
        newconn = self.zcore.getShortProfileForNetUpdateSelfConnection(netUpdateConn)
        window = NetworkUpdateSelfConnectionView( newconn, self.show_detail_profil,self.zcore)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
        
        

    def _send_request_invite_profile(self, profile, subject, body):
        
            self.zcore.sendConnectionRequestToProfile(profile, subject, body)
            self.show_banner_information("invitation sent")
            # remove  the view
            hildon.WindowStack.get_default().pop_1()
       


    def _send_message_to_profile(self, profile, subject, body):
        
            self.zcore.sendMessageToProfile(profile, subject, body)
            self.show_banner_information("message sent to the contact")
            # remove  the view
            hildon.WindowStack.get_default().pop_1()
     


    def _show_detail_profil(self, shortProfile):
    
        fullProfile = self.zcore.getFullProfileForShortProfile(shortProfile)
        self.show_full_profile(fullProfile)
   


    def _send_message_to_network(self, message):
       
        self.zcore.sendMessageToNetwork(message)
        self.show_banner_information("message sent to the network")
        # remove  the view
        hildon.WindowStack.get_default().pop_1()
        
    
    def _clear_status(self):
       
        self.zcore.clearMyStatus()
        self.show_banner_information("status cleared")
        # remove  the view
        hildon.WindowStack.get_default().pop_1()
       


    def _modify_status(self, status):
      
        self.zcore.setMyStatus(status)
        self.show_banner_information("status updated")
        # remove  the view
        hildon.WindowStack.get_default().pop_1()       
 
    def _update_network_news(self, view):
        netUpdate = self.zcore.getNetworkUpdate()
        view.update_network_news(netUpdate, self.zcore)   
          
    def _select_contact(self, person):
        fullProfile = self.zcore.getFullProfileForShortPerson(person)
        self.show_full_profile(fullProfile)     
    
    def _show_update_status(self):
        setStatus = self.zcore.getAllMyStatus()
        window = UpdateStatusView(setStatus, self.modify_status, self.clear_status)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()



    def _unavailable_handler(self):
        '''
        Do appropriate action when the core throw an UnvailableException.
        Currently simply display a banner message 
        '''
        return self.show_banner_information("The service is unavailable")

    def _configuration_required_handler(self):
        '''
        Do appropriate action when the core throw a ConfigurationRequieredException.
        Currently simply display a banner message
        '''
        return self.show_banner_information("The service need to be configured")
    
    def _default_exception_handler(self):
        '''
        Do appropriate action when the core throw a Exception.
        Currently simply display a banner message.
        '''
        #TODO open a window with the stack trace and ask to send the bug report            
        logging.exception("!!! UNHANDLED EXCEPTION !!!")      
        self.show_banner_information("An exception occured, please report the bug")
        type, value,tb = sys.exc_info() 
        window = BugReportView(type, value, tb, self.submit_issue)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
                        
        

    def _call_handled_method(self, method, *arg):
        try:
            method(*arg)
        except plugin.UnvailableException:
            self._unavailable_handler()
        except plugin.ConfigurationRequiredExeption:
            self._configuration_required_handler()
        except :
            # this is an unkown exception
            self._default_exception_handler()


    def _reset_cache(self):
        self.zcore.resetCacheData()
        self.show_banner_information("cache cleared")
    
    def submit_issue(self, title, description):
        # check credential
        try:
            parent = hildon.WindowStack.get_default().peek()
            dialog = hildon.LoginDialog(parent)
            dialog.set_message("Gmail account required")            
            response = dialog.run()
            username = dialog.get_username()
            password = dialog.get_password()
            dialog.destroy()
            if response == gtk.RESPONSE_OK:
                try:
                    issues_client = gdata.projecthosting.client.ProjectHostingClient()
                    issues_client.client_login(username, password,"zourite", "code")
                    versionInstance = version.getInstance()
                    versionStr = versionInstance.getVersion()
                    revisionStr = versionInstance.getRevision()
                    labels = ['Type-Defect', 'Priority-High', 'Version-' + versionStr, 'Revision-' + revisionStr]
                    issues_client.add_issue("zourite", title, description, "tbressure", labels=labels)
                except:                    
                    self.show_banner_information("failed to send issue")
                    logging.exception("Failed to report the previous issue due to")
                else:
                    self.show_banner_information("issue sent")
            else:
                self.show_banner_information("bug report cancelled")
        finally:
            hildon.WindowStack.get_default().pop_1()

    def show_full_profile(self, fullProfile):
        window = FullProfileView(fullProfile, self.show_invite_profile, self.show_message_profile, self.zcore)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()


 
    def init_menu(self, window):
        '''
        create menu for all windows that will be used
        in this program
        '''
        menu = hildon.AppMenu()
           
        
        refreshMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        refreshMenuBtn.set_label("Refresh");
        refreshMenuBtn.connect("clicked", self.refresh_from_menu, None)
        menu.append(refreshMenuBtn)
        
        contactMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        contactMenuBtn.set_label("Contact");
        contactMenuBtn.connect("clicked", self.show_contact_list, None)
        menu.append(contactMenuBtn)   
        
        newsMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        newsMenuBtn.set_label("News");
        newsMenuBtn.connect("clicked", self.show_network_news_from_menu, None)
        menu.append(newsMenuBtn)            
                
        paramMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        paramMenuBtn.set_label("Settings");
        paramMenuBtn.connect("clicked", self.show_settings_from_menu, None)
        menu.append(paramMenuBtn)
    
        aboutMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        aboutMenuBtn.set_label("About");
        aboutMenuBtn.connect("clicked", self.show_about_dialog, None)
        menu.append(aboutMenuBtn)   
        
        menu.show_all()
        window.set_app_menu(menu)  
           
           
    def show_contact_list(self, widget, data):
        '''
        show the contact list window
        '''
        self._call_handled_method(self._show_contact_list)

    def select_contact(self, person):
        '''
        Bring up a gui showing a contact
        '''
        self._call_handled_method(self._select_contact, person)
        
    
    def show_settings_from_menu(self, widget, data):
        '''
        Show settings dialog
        '''
        window = SettingsView(self.save_settings, self.restore_default_settings, self.reset_cache, self.zcore)
        
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
        
    def reset_cache(self):
        '''
        remove any cached data
        '''
        self._call_handled_method(self._reset_cache)
        
    def save_settings(self, settings):
        self._call_handled_method(self._save_settings,settings)

    def restore_default_settings(self):
        self._call_handled_method(self._restore_default_settings)
    
    def show_about_dialog(self, widget, data):
       '''
       Show an information dialog about the program
       '''
       window = AboutView()
        
       self.program.add_window(window)
       window.show_all()
      
       
    def refresh_from_menu(self, widget, data):
        self._call_handled_method(self._refresh_from_menu)
       
    def show_network_news_from_menu(self, widget, data):
        self.show_network_news()
       
    def show_note_information(self, mess):
       parent = hildon.WindowStack.get_default().peek()
       note = hildon.hildon_note_new_information(parent,mess)                 
   
       response = gtk.Dialog.run(note)

    def show_banner_information(self, mess):
       parent = hildon.WindowStack.get_default().peek()
       banner= hildon.hildon_banner_show_information(parent,"", mess)
       banner.set_timeout(2000) 
    
    def init_main_view(self):
        '''
        create a new window for the main view of the application
        '''
       
        # display a spashscreen
        window = SplashScreenView()       
        self.init_menu(window)
        self.program.add_window(window)       
        window.connect("destroy", gtk.main_quit, None)
        # This call show the window and also add the window to the stack
        window.show_all()

        # TODO do some stuff here                
        self.show_network_news()                            
       
       

   

    def show_network_news(self):
        '''
        ZouriteGui always display the network news as main view.
        '''
        # Create the home view        
        window = NetworkNewsView(self.update_network_news, self.select_an_update_network_new, self.show_update_status, self.show_post_news)        
        window.set_title("Network news")
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
                
       

   
    def update_network_news(self, view):
        '''
        callback function invoked by the NetworkNewsView when it want
        to be refreshed
        view is the NetworkNewsView
        '''
        self._call_handled_method(self._update_network_news,view)
        
        
    def show_update_status(self, widget, view):
        # TODO this method must be called by the NetworkNewsView not by gtk loop
        # in order to avoid theses extra unused parameter
        self._call_handled_method(self._show_update_status)
        
            
    
    
    def modify_status(self, status):
        '''
        callback funtion to modify the user status
        '''
        self._call_handled_method(self._modify_status,status)
            
        
        
    def clear_status(self):
        '''
        callback funtion to modify the user status
        '''
        self._call_handled_method(self._clear_status)
            
    
    def show_post_news(self, widget, view):
        '''
        Show a view to edit the message to post
        '''
        window = NewsMessageEditorView(self.send_message_to_network)
        
        
        self.init_menu(window)
           
        self.program.add_window(window)
        window.show_all()
        
    def send_message_to_network(self, message):
        '''
        Send the message and close the current view
        '''
        self._call_handled_method(self._send_message_to_network,message)
    
    def show_detail_profil(self, widget, shortProfile):
        '''
        A Windows call us to display the full profile for the given short profile
        '''
        self._call_handled_method(self._show_detail_profil,shortProfile)

    
    def place_call(self, person):
        '''
        Initiate a phone call to the given ShortProfile.
        '''
        # this launch a call so we must have aggreement of the user
        message = "Do you want to call " + person.firstname + " " + person.lastname + " at " + person.phone
        parent = hildon.WindowStack.get_default().peek()
        note = hildon.hildon_note_new_confirmation(parent, message)
   
        response = gtk.Dialog.run(note)
   
        note.destroy()
        
        if response == gtk.RESPONSE_OK:        
                       
            bus = dbus.SystemBus()
            csd_call = dbus.Interface(bus.get_object('com.nokia.csd',
                                                     '/com/nokia/csd/call'),
                                                     'com.nokia.csd.Call')
            csd_call.CreateWith(person.phone, dbus.UInt32(0))
            self.show_banner_information("call in progress")
        else:
            self.show_banner_information("call cancelled")
        
            
        

    def place_email(self, person):
        '''
        Initiate a mail editor to create a message to the given ShortProfile
        '''
        bus = dbus.SystemBus()
        csd_call = dbus.Interface(bus.get_object('com.nokia.modest',
                                            '/com/nokia/modest'),
                                            'com.nokia.modest')
        csd_call.MailTo("mailto:" + person.email)
    
    
    def show_invite_profile(self, widget, shortProfile):
        '''
        The widget call us to invite the given profile
        '''        
        window = MessageEditorView(shortProfile, self.send_request_invite_profile)
        
        
        self.init_menu(window)
           
        self.program.add_window(window)
        window.show_all()
        
    def show_message_profile(self, widget, shortProfile):
        '''
        The widget call us to message the given profile
        '''
        
        window = MessageEditorView(shortProfile, self.send_message_to_profile)
        
        
        self.init_menu(window)
           
        self.program.add_window(window)
        window.show_all()
    
    def send_message_to_profile(self, profile, subject, body):
        '''
        The send bouton of the view MessageEditorView has been clicked.
        Our job i snow to send the message ! 
        '''       
        self._call_handled_method(self._send_message_to_profile,profile, subject, body)

    
    def send_request_invite_profile(self, profile, subject, body):
        '''
        The send bouton of the view MessageEditorView has been clicked.
        Our job is now to send the invitation !
        '''
        self._call_handled_method(self._send_request_invite_profile,profile, subject, body)
    
    def select_an_update_network_new(self, netUpdateConn):
        '''
        The user has selected a network update news. We have to show the detail now.
        '''
        # Create the news news detail
        
       
        if isinstance(netUpdateConn, plugin.NetworkUpdateSelfConnection) :     
            self.show_network_update_self_connection_detail(netUpdateConn)
                            
        elif isinstance(netUpdateConn, plugin.NetworkUpdateConnection) :
            self.show_network_update_connection_detail(netUpdateConn)
        else:
            self.show_note_information("the type " + str(netUpdateConn.type) + " is not supported yet")            
        
        # This call show the window and also add the window to the stack
        
    
    def show_network_update_connection_detail(self, netUpdateConn):
        '''
        SHow the detail of a network update connection 
        '''
        self._call_handled_method(self._show_network_update_connection_detail,netUpdateConn)
     
    def show_network_update_self_connection_detail(self, netUpdateConn):
        '''
        SHow the detail of a network update connection 
        '''
        self._call_handled_method(self._show_network_update_self_connection_detail,netUpdateConn)
   
     
    
    def run(self):
        # the splash screen  is displayed,now show the home screen in a fancy tansition               
        gtk.main()


        

class ZouriteStackableWindow(hildon.StackableWindow):
    '''
    general design of any GUI of this app
    '''
    
    centerview = None;
    bottomButtons = None;
    
    def __init__(self, title="Zourite"):
        hildon.StackableWindow.__init__(self)
        self.set_title(title)
   
        container = gtk.VBox();                
                  
        self.centerview = gtk.VBox() 
        
        self.init_center_view(self.centerview)
        
        pannable_area = hildon.PannableArea()
        pannable_area.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
        pannable_area.add_with_viewport(self.centerview)
        
        self.bottomButtons = gtk.HBox(homogeneous=True)
        
        self.init_bottom_button(self.bottomButtons)
        

        bottomAlign = gtk.Alignment(0.5,0,0,0)
        bottomAlign.add(self.bottomButtons)
        
        container.pack_start(pannable_area)
        container.pack_start(bottomAlign, expand=False)
        
        self.add(container)

    def init_center_view(self, centerview):
        '''
        This hook should be implemented by subclass to
        add widget in the centerview which is both instance
        and attribute variable for convenience
        '''
        pass;
    
    def init_bottom_button(self, bottomButtons):
        '''
        This hook should be implemented by subclass to
        add button in the bottom button which is both instance
        and attribute variable for convenience
        '''
        pass;

    def add_button(self, button):
        '''
        goodies to add a button to this view
        '''
        self.bottomButtons.pack_start(button,True,True,0)
    
    def create_button(self, name, value=None):
        '''
        goodies to create a button for this view
        ''' 
        return hildon.Button(gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, name, value)

    def justifyLeft(self, widget):
          leftAlign = gtk.Alignment(0,0,0,0)
          leftAlign.add(widget)
          return leftAlign

class AboutView(ZouriteStackableWindow):
    '''
    This view show a fancy about dialog
    '''

    def init_center_view(self, centerview):
        
        pixbuf = gtk.gdk.pixbuf_new_from_file("zourite-logo.png")
        image = gtk.Image()
        image.set_from_pixbuf(pixbuf)
        centerview.add(image)
        
        centerview.add(gtk.Label("ZOURITE - " + version.getInstance().getVersion()))
        centerview.add(gtk.Label("Professional networking application for N900"))
        centerview.add(gtk.Label("by Thierry Bressure - http://blog.zourite.bressure.net"))


    def init_bottom_button(self, bottomButtons):
        blogButton = self.create_button("Blog")
        blogButton.connect("clicked", self.on_blog_clicked_event, None)
        self.add_button(blogButton)
        
        siteButton = self.create_button("Site")
        siteButton.connect("clicked", self.on_site_clicked_event, None)
        self.add_button(siteButton)
        
        groupsiteButton = self.create_button("Groups")
        groupsiteButton.connect("clicked", self.on_group_clicked_event, None)
        self.add_button(groupsiteButton)

    def on_blog_clicked_event(self, widget, data):
         webbrowser.open_new_tab("http://blog.zourite.bressure.net");
    
    def on_site_clicked_event(self, widget, data):
         webbrowser.open_new_tab("http://zourite.bressure.net");
         
    def on_group_clicked_event(self, widget, data):
         webbrowser.open_new_tab("http://group.zourite.bressure.net");
             
    

class SplashScreenView(ZouriteStackableWindow):
    '''
    This is the first view of the application e.g. the main view.  
    '''

    def init_center_view(self, centerview):
        pixbuf = gtk.gdk.pixbuf_new_from_file("zourite-logo.png")
        for i in range(1,4):
            hbox = gtk.HBox()
            for j in range(1,5):                
                image = gtk.Image()
                image.set_from_pixbuf(pixbuf)
                hbox.add(image)
            centerview.add(hbox)


class BugReportView(ZouriteStackableWindow):
    '''
    This view show the bug and give the user the opportunity to report it
    '''
    type = None
    value = None
    traceback = None
    
    submit_issue_callback = None
    
    _body = None
    _subject = None
    
    def __init__(self, type, value, traceback, submit_issue_callback):
        self.type = type
        self.value = value
        self.traceback = traceback
        self.submit_issue_callback = submit_issue_callback
        ZouriteStackableWindow.__init__(self, title="Bug reporting") 

    def init_center_view(self, centerview):
       
        subjectLbl = gtk.Label("Subject")
        centerview.pack_start(self.justifyLeft(subjectLbl), False)
        self._subject = hildon.Entry(gtk.HILDON_SIZE_FULLSCREEN_WIDTH)
        self._subject.set_placeholder("enter a subject")
        self._subject.set_text(str(self.type) + " : " + str(self.value))
        centerview.pack_start(self._subject, False)
        contentLbl = gtk.Label("Content")
        centerview.pack_start(self.justifyLeft(contentLbl), False)
        self._body = hildon.TextView()
        self._body.set_placeholder("enter the message here")
        self._body.set_wrap_mode(gtk.WRAP_WORD)
        stacktrace = traceback.format_exception(self.type, self.value, self.traceback)
        buf = self._body.get_buffer()
        for line in stacktrace:
            end = buf.get_end_iter()
            buf.insert(end, line, len(line))
        centerview.add(self._body)
        return ZouriteStackableWindow.init_center_view(self, centerview)


    def init_bottom_button(self, bottomButtons):
        post = self.create_button("Post issue", None)
        post.connect("clicked", self.on_post_button_clicked, self)
        self.add_button(post)                     
        return ZouriteStackableWindow.init_bottom_button(self, bottomButtons)

    def on_post_button_clicked(self, widget, view):
        buffer = view._body.get_buffer()
        body = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
        subject =  view._subject.get_text()
        self.submit_issue_callback(subject, body)
        




class ContactListView(ZouriteStackableWindow):
    '''
    View to browse all contact
    '''
    
    def __init__(self, select_contact_callback, phone_call_callback, email_callback, zcore):
        self.zcore= zcore        
        self.select_contact_callback = select_contact_callback
        self.phone_call_callback = phone_call_callback            
        self.email_callback = email_callback
        
        self.email_pixbuf = gtk.gdk.pixbuf_new_from_file("email_message_editor.png")
        self.email_pixbuf_disabled = self.email_pixbuf.copy()
        self.email_pixbuf_disabled.saturate_and_pixelate(self.email_pixbuf_disabled, 0.3, False)   
        
        self.call_pixbuf = gtk.gdk.pixbuf_new_from_file("general_call.png")
        self.call_pixbuf_disabeld = self.call_pixbuf.copy()
        self.call_pixbuf_disabeld.saturate_and_pixelate(self.call_pixbuf_disabeld, 0.3, False)
        
        # TODO create a list of contact
        ZouriteStackableWindow.__init__(self, "Contacts list")

    def init_center_view(self, centerview):
        
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str,gtk.gdk.Pixbuf, gtk.gdk.Pixbuf,object)
        # sort on contacts name
        self.model.set_sort_column_id(1,gtk.SORT_ASCENDING)
        
        self.refresh()
         # create the view
        self.view = gtk.TreeView(self.model)
        
        picture_column = gtk.TreeViewColumn("picture")
        picture_column_renderer = gtk.CellRendererPixbuf()
        picture_column.pack_start(picture_column_renderer)
        picture_column.set_attributes(picture_column_renderer, pixbuf=0) 
        self.view.append_column(picture_column)
        
        contact_column = gtk.TreeViewColumn("contact")
        contact_column.set_property("sizing", gtk.TREE_VIEW_COLUMN_FIXED)
        contact_column.set_property("fixed-width", 300)
        contact_column_renderer = gtk.CellRendererText()
        contact_column.pack_start(contact_column_renderer)
        contact_column.set_attributes(contact_column_renderer, text=1)
        self.view.append_column(contact_column)

        headline_column = gtk.TreeViewColumn("headline")
        headline_column.set_property("sizing", gtk.TREE_VIEW_COLUMN_FIXED)
        headline_column.set_property("fixed-width", 300)       
        renderer_txt = gtk.CellRendererText()
        renderer_txt.set_property("size-points",10)
        headline_column.pack_start(renderer_txt)
        headline_column.set_attributes(renderer_txt, text=2)
        self.view.append_column(headline_column)

        emailcolumn = gtk.TreeViewColumn("email")            
        emailcolumn_renderer = gtk.CellRendererPixbuf()
        emailcolumn.pack_start(emailcolumn_renderer)
        emailcolumn.set_attributes(emailcolumn_renderer, pixbuf=3) 
        self.view.append_column(emailcolumn)
        
        callcolumn = gtk.TreeViewColumn("call")            
        callcolumn_renderer = gtk.CellRendererPixbuf()
        callcolumn.pack_start(callcolumn_renderer)
        callcolumn.set_attributes(callcolumn_renderer, pixbuf=4) 
        self.view.append_column(callcolumn)

        self.view.connect("row-activated", self.on_contact_row_activated, None)
        

        
        
        centerview.add(self.view)
        

    def on_contact_row_activated(self,  treeview, path, view_column,  user_data):
        store = treeview.get_model()
        iter = store.get_iter(path)
        person, = store.get(iter,5)
        # check if the activation come from a nice icon
        title = view_column.get_property('title')
        if  title == "call" and person.phone is not None:
             self.phone_call_callback(person)
        elif title == "email" and person.email is not None:
             self.email_callback(person)
        else:            
            self.select_contact_callback(person)

 
        

    def init_bottom_button(self, bottomButtons):
        return ZouriteStackableWindow.init_bottom_button(self, bottomButtons)

    def refresh(self):
        '''
        this view is asked to reresh itself with latest data.
        the refresh function is defined at instanciation
        '''
        peopleList = self.zcore.getMyConnections()
        self.model.clear()
        
        # simply push all data in the model            
        for people in peopleList:
                contact = people.lastname.upper() + " " + people.firstname.capitalize()
                headline = people.headline
                                
                if people.email is None:
                    emailPict = self.email_pixbuf_disabled
                else:
                    emailPict = self.email_pixbuf
                if people.phone is None:
                    phonePict = self.call_pixbuf_disabeld
                else:
                    phonePict = self.call_pixbuf
                    
                self.model.append([self.zcore.getGtkPixBufForProfile(people).scale_simple(80,80,gtk.gdk.INTERP_BILINEAR) , 
                                   contact, headline, 
                                   emailPict,
                                   phonePict
                                   , people])
        
       

class SettingsView(ZouriteStackableWindow):
    '''
    View to manage settings of the application.
    '''
    
    def __init__(self, save_callback, default_callback, reset_cache_callback, zcore):        
        self.zcore = zcore
        self.save_callback = save_callback
        self.default_callback = default_callback
        self.reset_cache_callback = reset_cache_callback
        self.gtkWidgetForPluginConf = {}
        ZouriteStackableWindow.__init__(self, "Settings")

    def _create_config_plugin_page(self, notebook, current_page_number):
        
        #todo change the notebook
        nextPage = gtk.VBox()
        # print current value
        keys = self.pluginConfData.keys()
        if plugin.PLUGIN_CONF_INFO in keys:
            nextPage.add(gtk.Label(self.pluginConfData[plugin.PLUGIN_CONF_INFO]))
        
        if plugin.PLUGIN_CONF_WEBBROWSER in keys:
            url = self.pluginConfData[plugin.PLUGIN_CONF_WEBBROWSER]
            nextPage.add(gtk.Label("please visit " + url))
            webbrowser.open_new_tab(url)
        
        self.gtkWidgetForPluginConf.clear()
        for key in keys:
        # handle special parameter
            if not key.startswith("__"):
                # standar parameter
                if self.pluginConfData[key] is None:
                    entry = hildon.Entry(gtk.HILDON_SIZE_AUTO_WIDTH)
                    self.gtkWidgetForPluginConf[key]=entry                    
                    caption = hildon.Caption(None, key, entry, None, hildon.CAPTION_MANDATORY)
                    nextPage.add(caption)
                
            
        
        notebook.insert_page(nextPage, position=current_page_number+1)


    def init_center_view(self, centerview):
        
        cacheSettings = gtk.HBox()
        
        cachePickerBtn =  hildon.PickerButton(gtk.HILDON_SIZE_AUTO , hildon.BUTTON_ARRANGEMENT_VERTICAL)
        cachePickerBtn.set_title("cache strategy")
        self.strategySelector = self.create_cache_strategy_selector()
        cachePickerBtn.set_selector(self.strategySelector)
        cacheSettings.add(cachePickerBtn)
             
        
        storePickerBtn =  hildon.PickerButton(gtk.HILDON_SIZE_AUTO , hildon.BUTTON_ARRANGEMENT_VERTICAL)
        storePickerBtn.set_title("cache store")
        self.storeSelector = self.create_cache_store_selector()
        storePickerBtn.set_selector(self.storeSelector)
        cacheSettings.add(storePickerBtn)
        
        centerview.add(cacheSettings)         
        
        resetCacheBtn = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, "clear cache", None)
        resetCacheBtn.connect("clicked", self.on_reset_cache_clicked_event, None)
        centerview.pack_start(resetCacheBtn, False, False, 0)
                
        pluginPickerBtn =  hildon.PickerButton(gtk.HILDON_SIZE_AUTO , hildon.BUTTON_ARRANGEMENT_VERTICAL)
        pluginPickerBtn.set_title("network")
        self.pluginSelector = self.create_plugin_selector()
        pluginPickerBtn.set_selector(self.pluginSelector)
        centerview.add(pluginPickerBtn)       
        
        plugigConfButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, "configure", None)
        plugigConfButton.connect("clicked", self.on_configure_plugin_clicked_event, None)
        centerview.pack_start(plugigConfButton, False, False, 0)
        
        
    def on_configure_plugin_clicked_event(self, widget, data):
        '''
        Open a dialog to choose the plugin to configure
        '''        
        self.gtkWidgetForPluginConf.clear()     
        
        notebook = gtk.Notebook()
        
        welcome = gtk.Label("This wizzard will help you to configure a plugin")        
        plugin_selector = self.create_plugin_selector_for_configuration()
        finish  = gtk.Label("The plugin is successfuly configured")
        
        notebook.append_page(welcome)
        notebook.append_page(plugin_selector,gtk.Label("select plugin"))
        notebook.append_page(finish)
        
        dialog = hildon.WizardDialog(self, "Plugin Configuration", notebook)
   
        # Set a handler for "switch-page" signal
        notebook.connect("switch-page", self.on_page_switch, dialog)
        notebook.connect("change-current-page", self.on_current_page_change, dialog)
        notebook.connect("page-added", self.on_page_added, dialog)
   
        # Set a function to decide if user can go to next page
        dialog.set_forward_page_func(self.some_page_func)
   
        dialog.show_all()
        response = dialog.run()
        dialog.destroy()
        
        self.gtkWidgetForPluginConf.clear()
        
    
    def  on_page_added(self, notebook, child, page_num, dialog):
        print >>sys.stderr, "page added num %d" % page_num               
        dialog.show_all()

    def on_current_page_change(self, notebook, offset, dialog):
        print >>sys.stderr, "current page change offset %d" % offset
        
    def on_page_switch(self, notebook, page, num, dialog):
       print >>sys.stderr, "Page swith %s %d" % (str(page), num)
       return True
   
    def some_page_func(self, notebook, current_page_number, userdata):   
        print >>sys.stderr, "check forward page for page " + str(current_page_number)
        if current_page_number == 0 :
            return True
        elif current_page_number == 1 :
            # must have choose a plugin
            plugin_selector = notebook.get_nth_page(current_page_number)            
            selectedRows = plugin_selector.get_selected_rows(0)
            if len(selectedRows) == 1 :
                # start the plugin configuration
                plugPath = selectedRows[0]
                plugModel =  plugin_selector.get_model(0)
                plugIter = plugModel.get_iter(plugPath)
                plugid, = plugModel.get(plugIter,0)                
                self.pluginConf = self.zcore.getPluginAmongAvailable(plugid)
                self.pluginConfData = self.pluginConf.configure()   
                self._create_config_plugin_page(notebook, current_page_number)
                notebook.next_page()
                return False
            else:
                # cannot jump to next page without selecting one plugin to configure
                return False
        else:
            # does the plugin expect some data ?
            if len(self.pluginConfData) > 0:
                # provide the data to the plugin
                
                for key in self.gtkWidgetForPluginConf.keys():
                    entry = self.gtkWidgetForPluginConf[key]
                    self.pluginConfData[key] = entry.get_text()
                self.gtkWidgetForPluginConf.clear()    
                self.pluginConfData = self.pluginConf.configure(self.pluginConfData)
                # prepare the next page if required by the plugin
                if len(self.pluginConfData) > 0:
                    self._create_config_plugin_page(notebook, current_page_number)
                    notebook.next_page()
                    return False
                else :
                    return True                    
            else:
                # no more data required by the plugin
                return True
            

    
    def on_reset_cache_clicked_event(self, widget, data):
        message = "This will remove all local data from your network. Are you sur?"
        parent = hildon.WindowStack.get_default().peek()
        note = hildon.hildon_note_new_confirmation(parent, message)
   
        response = gtk.Dialog.run(note)
   
        note.destroy()
        
        if response == gtk.RESPONSE_OK:
            self.reset_cache_callback()      
    
    
    def create_plugin_selector_for_configuration(self):
        #Create a HildonTouchSelector with a single text column
        selector = hildon.TouchSelector(text = True)
        # Populate selector    
        pluginPathList = self.pluginSelector.get_selected_rows(0)
        pluginModel = self.pluginSelector.get_model(0)
        
        for path in pluginPathList:
                    
            pluginIter = pluginModel.get_iter(path)
            pluginId, = pluginModel.get(pluginIter,0)            
            selector.append_text(pluginId)

        return selector
    
    
    def create_plugin_selector(self):
        #Create a HildonTouchSelector with a single text column
        selector = hildon.TouchSelector(text = True)
        # Set selection mode to allow multiple selection
        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)  
        registeredPlugin = self.zcore.getRegisteredPlugin()
        registered_plugin_id_list = map(lambda(x): x.get_plugin_id(),registeredPlugin)
        # Populate selector
        rang = -1
        for plugin in self.zcore.getAvailablePlugins():
            rang = rang + 1
            label = plugin.get_plugin_id() # TODO replace by a user friendly label
            # Add item to the column
            selector.append_text(label)
            # must this be selected ?
            plugin_id = plugin.get_plugin_id()
             
            if plugin_id in registered_plugin_id_list:
                selector.select_iter(0, selector.get_model(0).get_iter(rang), False)
        
    
        
        return selector
    
    
    def create_cache_strategy_selector(self):
        #Create a HildonTouchSelector with a single text column
        selector = hildon.TouchSelector(text = True)
    
        currentCacheStrategy = self.zcore.getCurrentCacheStrategy()
        
        # Populate selector
        rang = -1
        for strategy in self.zcore.getAvailableCacheStrategies():
            rang = rang + 1
            label = strategy # TODO replace by a user friendly label
            # Add item to the column
            selector.append_text(label)
            if strategy == currentCacheStrategy :
                selector.select_iter(0,selector.get_model(0).get_iter(rang), False)
    
        # Set selection mode to allow multiple selection
        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
    
        return selector
    
    
    
    def create_cache_store_selector(self):
        #Create a HildonTouchSelector with a single text column
        selector = hildon.TouchSelector(text = True)
    
        currentCacheStore = self.zcore.getCurrentCacheStorage()
        
        # Populate selector
        rang = -1
        for store in self.zcore.getAvailableCacheStore():
            rang = rang + 1
            label = store  # TODO replace by a user friendly label
            # Add item to the column
            selector.append_text(label)
            if store == currentCacheStore :
                selector.select_iter(0,selector.get_model(0).get_iter(rang), False)
    
        # Set selection mode to allow multiple selection
        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
    
        return selector
    
   

    def init_bottom_button(self, bottomButtons):
        saveBtn = self.create_button("Save")
        saveBtn.connect("clicked", self.on_save_clicked, self)
        self.add_button(saveBtn)
        defaultBtn = self.create_button("Default")
        defaultBtn.connect("clicked", self.on_default_clicked, self)
        self.add_button(defaultBtn)
      
    def on_save_clicked(self, widget, data):
        settings = zourite.Settings()
        
        stratPath = self.strategySelector.get_selected_rows(0)[0]
        stratModel =  self.strategySelector.get_model(0)
        stratIter = stratModel.get_iter(stratPath)
        settings.cache_strategy, = stratModel.get(stratIter,0)
        
        storePath = self.storeSelector.get_selected_rows(0)[0]
        storeModel =  self.storeSelector.get_model(0)
        storeIter = storeModel.get_iter(storePath)
        settings.cache_storage, = storeModel.get(storeIter,0)
        
        pluginPathList = self.pluginSelector.get_selected_rows(0)
        pluginModel = self.pluginSelector.get_model(0)
        for path in pluginPathList:
           
            pluginIter = pluginModel.get_iter(path)
            pluginId, = pluginModel.get(pluginIter,0)
            settings.registered_plugin_ids.append(pluginId)
        
        self.save_callback(settings)

    def on_default_clicked(self, widget, data):
        self.default_callback()
    

class UpdateStatusView(ZouriteStackableWindow):
    '''
    View used to update the user statuzs status
    '''
    modify_status_callback = None
    clear_status_callback = None
    statusEntry = None
    def __init__(self, currentStatus, modify_status_callback, clear_status_callback):
        '''
        currentStatus a set of current status
        '''
        self.currentStatus = currentStatus
        self.modify_status_callback = modify_status_callback
        self.clear_status_callback = clear_status_callback
        
        ZouriteStackableWindow.__init__(self, "Status")

    def init_center_view(self, centerview):
        
        tableView = gtk.Table(2, 2, True)
                
        tableView.attach(self.justifyRight(gtk.Label("Current status")),0,1,0,1, xpadding=5)
        # construct a status
        statusStr = ""
        for status in self.currentStatus:
            if len(statusStr) > 0:
                statusStr = statusStr + " "
            statusStr = statusStr + status
        if len(statusStr) == 0:
            statusStr = "no status defined"
        tableView.attach(self.justifyLeft(gtk.Label(statusStr)),1,2,0,1)
        tableView.attach(self.justifyRight(gtk.Label("New status")),0,1,1,2, xpadding=5)
        self.statusEntry = hildon.Entry(gtk.HILDON_SIZE_HALFSCREEN_WIDTH)
        tableView.attach(self.justifyLeft(self.statusEntry),1,2,1,2)        
        
        centerview.add(tableView)


    def init_bottom_button(self, bottomButtons):
        modifyBtn = self.create_button("Modify status")
        modifyBtn.connect("clicked", self.on_modify_button_clicked, self.statusEntry)
        self.add_button(modifyBtn)
        clearBtn = self.create_button("Clear status")
        clearBtn.connect("clicked", self.on_clear_button_clicked, None)
        self.add_button(clearBtn)
        
    def on_modify_button_clicked(self, widget, entry):
        self.modify_status_callback(entry.get_text())
    
    def on_clear_button_clicked(self, widget, data):
        self.clear_status_callback()
        

    def justifyLeft(self, widget):
          leftAlign = gtk.Alignment(0,0.5,0,0)
          leftAlign.add(widget)
          return leftAlign

    def justifyRight(self, widget):
          leftAlign = gtk.Alignment(1,0.5,0,0)
          leftAlign.add(widget)
          return leftAlign


    
class NewsMessageEditorView(ZouriteStackableWindow):
    '''
    View used to edit a news message for all connections in my network
    '''
    
    body = None  
    
    def __init__(self, on_send_bouton_clicked_callback):
    
        self.on_send_bouton_clicked_callback = on_send_bouton_clicked_callback
        ZouriteStackableWindow.__init__(self, title = "Edit a message")        
                

    def init_bottom_button(self,  hbox):
        sendBtn = self.create_button("Send")
        sendBtn.connect("clicked", self.on_send_bouton_clicked, self.body)
        self.add_button(sendBtn)
        
    def on_send_bouton_clicked(self, widget, textView):
        buffer = textView.get_buffer()
        body = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
        self.on_send_bouton_clicked_callback(body)

    def init_center_view(self, centerview):
        
        # create the content of the cneter view
      
        contentLbl = gtk.Label("Content")
        centerview.pack_start(self.justifyLeft(contentLbl), False)
        self.body = hildon.TextView()
        self.body.set_placeholder("enter the message here")
        self.body.set_wrap_mode(gtk.WRAP_WORD)
        centerview.add(self.body)

            

class MessageEditorView(ZouriteStackableWindow):
    '''
    View used to edit a message
    '''
    
    subject = None
    body = None
    profile = None
    
    def __init__(self, shortProfile, send_bouton_callback):
        self.profile = shortProfile
        self.send_bouton_callback = send_bouton_callback
        ZouriteStackableWindow.__init__(self, title = "Edit a message")        
                

    def init_bottom_button(self,  hbox):
        sendBtn = self.create_button("Send")
        sendBtn.connect("clicked", self.on_send_button_clicked, self)
        self.add_button(sendBtn)

    def on_send_button_clicked(self, widget, view):
        buffer = view.body.get_buffer()
        body = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
        subject =  view.subject.get_text()
        self.send_bouton_callback(self.profile, subject, body)

    def init_center_view(self, centerview):
        
        # create the content of the cneter view
        subjectLbl = gtk.Label("Subject")
        centerview.pack_start(self.justifyLeft(subjectLbl), False)
        self.subject = hildon.Entry(gtk.HILDON_SIZE_FULLSCREEN_WIDTH)
        self.subject.set_placeholder("enter a subject")
        centerview.pack_start(self.subject, False)
        contentLbl = gtk.Label("Content")
        centerview.pack_start(self.justifyLeft(contentLbl), False)
        self.body = hildon.TextView()
        self.body.set_placeholder("enter the message here")
        self.body.set_wrap_mode(gtk.WRAP_WORD)
        centerview.add(self.body)

         
   


class FullProfileView(ZouriteStackableWindow):
    '''
    View showing the detail of a profile e.g full profile
    '''        
    def __init__(self, fullProfile, on_invite_button_clicked, on_message_button_clicked,  zcore):
        
        self.fullProfile = fullProfile
        self.on_invite_button_clicked = on_invite_button_clicked
        self.on_message_button_clicked = on_message_button_clicked
        self.zcore = zcore
        
        ZouriteStackableWindow.__init__(self, title = "Profile detail")        
    
     
    def init_bottom_button(self,  hbox):
        
        # TODO if profile is part of connection we can send him a message        
        # otherwose we should invite him
        messageBtn = self.create_button( "Send a message")
        messageBtn.connect("clicked", self.on_message_button_clicked, self.fullProfile)
        self.add_button(messageBtn)
        inviteBtn = self.create_button("Invite")
        inviteBtn.connect("clicked", self.on_invite_button_clicked, self.fullProfile)
        self.add_button(inviteBtn)


    def init_center_view(self, centerview ):
        centerview.set_spacing(5)
        # add the header
        header = self.create_header(self.fullProfile, self.zcore)
        centerview.pack_start(header, False)
        # add the summary
        summary = gtk.Label(self.fullProfile.summary)
        summary.set_line_wrap(True)
        frame = gtk.Frame("summary")
        frame.add(summary)
        centerview.pack_start(frame)
        # add the positions
        for position in self.fullProfile.positions:
            frame = gtk.Frame(position.title + " at " + position.company)
            if position.dateDebut is None:
                datePosition = ""
            else :
                datePosition = str(position.dateDebut)
            summary = gtk.Label("from " + datePosition + " : " + position.summary)
            summary.set_line_wrap(True)
            frame.add(summary)
            centerview.pack_start(frame)
        
        return centerview


    
        
    def create_header(self, fullProfile, zcore):
        header = gtk.HBox()
        image = zcore.getGtkImageForProfile(fullProfile)
        header.pack_start(image)
        description = gtk.VBox()       
        description.pack_start(self.justifyLeft(gtk.Label(fullProfile.lastname.upper())))
        description.pack_start(self.justifyLeft(gtk.Label(fullProfile.firstname.capitalize())))
        description.pack_start(self.justifyLeft(gtk.Label(fullProfile.headline)))
        description.pack_start(self.justifyLeft(gtk.Label(fullProfile.industry)))
        header.pack_start(description)
        return header
    
    def justifyLeft(self, widget):
          leftAlign = gtk.Alignment(0,0,0,0)
          leftAlign.add(widget)
          return leftAlign

class NetworkUpdateConnectionView(ZouriteStackableWindow):
    '''
    View showing an update connection details
    '''    
    def __init__(self, myConnShortProfile, newConnShortProfile, on_detail_profil_request, on_invite_bouton_clicked, zcore):
        
        self.myConnShortProfile = myConnShortProfile
        self.newConnShortProfile = newConnShortProfile
        self.on_detail_profil_request = on_detail_profil_request
        self.on_invite_bouton_clicked = on_invite_bouton_clicked
        self.zcore = zcore
        
        ZouriteStackableWindow.__init__(self, title = "New connection update")      
        
      
    def init_bottom_button(self, hbox):
        self.add_button(self.create_button("Make a comment"))
        inviteBtn = self.create_button( "Invite")
        inviteBtn.connect("clicked", self.on_invite_bouton_clicked, self.newConnShortProfile)
        self.add_button(inviteBtn)
       


    def init_center_view(self,  centerview):
        tableView = gtk.Table(3, 3, False)
        myconn_pict, myconn_name, myconn_lbl = self.create_shortProfile(self.myConnShortProfile, self.on_detail_profil_request, self.zcore)
        centralVBox = gtk.VBox()
        centralVBox.pack_start(gtk.Label("is now connected to"))
        newconn_pict, newconn_name, newconn_lbl = self.create_shortProfile(self.newConnShortProfile, self.on_detail_profil_request, self.zcore)
        tableView.attach(myconn_pict, 0, 1, 0, 1, xoptions=gtk.EXPAND, yoptions=gtk.SHRINK)
        tableView.attach(myconn_name, 0, 1, 1, 2)
        tableView.attach(myconn_lbl, 0, 1, 2, 3)
        tableView.attach(centralVBox, 1, 2, 0, 3, xoptions=gtk.EXPAND, yoptions=gtk.SHRINK | gtk.EXPAND)
        tableView.attach(newconn_pict, 2, 3, 0, 1, xoptions=gtk.EXPAND, yoptions=gtk.SHRINK)
        tableView.attach(newconn_name, 2, 3, 1, 2)
        tableView.attach(newconn_lbl, 2, 3, 2, 3)
        centerview.add(tableView)

        
    def create_shortProfile(self, shortProfile, on_detail_profil_request, zcore):
            
        
        
        picture = zcore.getGtkImageForProfile(shortProfile)
        pictAlign = gtk.Alignment(0.5,0.5,0,0)
        pictAlign.add(picture)
            
            
            
        nameButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT| gtk.HILDON_SIZE_AUTO_WIDTH , hildon.BUTTON_ARRANGEMENT_VERTICAL,shortProfile.firstname, shortProfile.lastname)
        nameButton.connect("clicked",on_detail_profil_request, shortProfile) 
        nameAlign = gtk.Alignment(0.5,0,0,0)
        nameAlign.add(nameButton)
            
            
        label = gtk.Label(shortProfile.headline)   
        label.set_single_line_mode(False)                
        label.set_max_width_chars(20)   
        label.set_line_wrap(True) # this does not work :-(
        labelAlign = gtk.Alignment(0.5,0,0,0)               
        labelAlign.add(label)
        
        return pictAlign, nameAlign, labelAlign    
        

class NetworkUpdateSelfConnectionView(ZouriteStackableWindow):
    '''
    View showing a new connection for the current user
    '''
    def __init__(self,  newConnShortProfile, on_detail_profil_request,  zcore):
        
       
        self.newConnShortProfile = newConnShortProfile
        self.on_detail_profil_request = on_detail_profil_request        
        self.zcore = zcore
        
        ZouriteStackableWindow.__init__(self, title = "New connection update")      
        
      
    def init_bottom_button(self, hbox):
        self.add_button(self.create_button("Make a comment"))
      
       


    def init_center_view(self,  centerview):
        centerview.add(gtk.Label("Your are  now connected to"))
        newconn_pict, newconn_name, newconn_lbl = self.create_shortProfile(self.newConnShortProfile, self.on_detail_profil_request, self.zcore)    
        centerview.add(newconn_pict)
        centerview.add(newconn_name)
        centerview.add(newconn_lbl)

        
    def create_shortProfile(self, shortProfile, on_detail_profil_request, zcore):
            
        
        
        picture = zcore.getGtkImageForProfile(shortProfile)
        pictAlign = gtk.Alignment(0.5,0.5,0,0)
        pictAlign.add(picture)
            
            
            
        nameButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT| gtk.HILDON_SIZE_AUTO_WIDTH , hildon.BUTTON_ARRANGEMENT_VERTICAL,shortProfile.firstname, shortProfile.lastname)
        nameButton.connect("clicked",on_detail_profil_request, shortProfile) 
        nameAlign = gtk.Alignment(0.5,0,0,0)
        nameAlign.add(nameButton)
            
            
        label = gtk.Label(shortProfile.headline)   
        label.set_single_line_mode(False)                
        label.set_max_width_chars(20)   
        label.set_line_wrap(True) # this does not work :-(
        labelAlign = gtk.Alignment(0.5,0,0,0)               
        labelAlign.add(label)
        
        return pictAlign, nameAlign, labelAlign   


        
class NetworkNewsView(ZouriteStackableWindow):
    
    model = None
    view = None
    refresh_func = None
    select_function = None
    
    def __init__(self, refresh_function, select_function, on_status_bouton_clicked, on_news_bouton_clicked):
        '''
        the refresh function is a pointer on the function which will be called
        when this views need to be updated with fresh data.
        the networkUpdateList is the initial data standing for the model of this list view
        '''
        self.refresh_func = refresh_function
        self.select_function = select_function
        self.on_status_bouton_clicked = on_status_bouton_clicked
        self.on_news_bouton_clicked = on_news_bouton_clicked

        ZouriteStackableWindow.__init__(self, title = "Network news")
    
       
                        

    def init_bottom_button(self, bottomButton):
        statusBtn = self.create_button("Update my status")
        statusBtn.connect("clicked", self.on_status_bouton_clicked, self)
        self.add_button(statusBtn)
        newsBtn = self.create_button("Post news")
        newsBtn.connect("clicked", self.on_news_bouton_clicked, self)
        self.add_button(newsBtn)


    def init_center_view(self, centerview):
        
        # create the model
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, object)
        self.refresh()
        # create the view
        self.view = gtk.TreeView(self.model)
        # create the column
        tvcolumnconn = gtk.TreeViewColumn("Connection")
        renderer_pix_conn = gtk.CellRendererPixbuf()
        tvcolumnconn.pack_start(renderer_pix_conn)
        tvcolumnconn.set_attributes(renderer_pix_conn, pixbuf=0) 
        self.view.append_column(tvcolumnconn)
        
        tvcolumntxt = gtk.TreeViewColumn("Network news")
        renderer_txt = gtk.CellRendererText()
        renderer_txt.set_property("size-points", 15)
        tvcolumntxt.pack_start(renderer_txt)
        tvcolumntxt.set_attributes(renderer_txt, text=1)
        self.view.append_column(tvcolumntxt)
       
        
        self.view.connect("row-activated", self.on_row_activated, None)
        centerview.add(self.view)


    def on_row_activated(self, treeview, path, view_column,  user_data):
         # TODO craete the view according to the type on news
        store = treeview.get_model()
        iter = store.get_iter(path)
        netUpdateConn, = store.get(iter,2)
        self.select_function(netUpdateConn)
    
       
    def refresh(self):
        '''
        this view is asked to refresh itself with latest data.
        the refresh function is defined at instanciation
        '''
        self.refresh_func(self)
    
    def update_network_news(self,networkUpdateLst, zcore):
        '''
        update the model of this view to reflect the network news in the list.
        '''
        self.model.clear()
        # simply push all data in the model
        for netUpd in networkUpdateLst:
                self.model.append([zcore.getGtkPixBufForNetwork(netUpd.networkId) ,str(netUpd), netUpd])
                
            

       




# -*- encoding: UTF-8 -*-

'''
Created on 9 mars 2010

@author: thierry
'''

import sys
import webbrowser

from zourite.core import zourite,plugin
from zourite.plugin.linkedin import linkedinplugin
from zourite.plugin.gmail import gmailplugin


from zourite.common import version


version.getInstance().submitRevision("$Revision: 159 $")


class AllPluginBootstrap():
    PLUGIN_LIST = [linkedinplugin.LinkedInPlugin(), gmailplugin.GmailPlugin()]
    
    def boot(self):
        zourite.ZOURITE_AVAILABLE_PLUGIN = self.PLUGIN_LIST



def runGui(argline=[]):
    
    from zourite.gui.zouriteGui import ZouriteGui 
    
    app = None
    if len(argline) > 1:
        if argline[1] == "mock":
            app = ZouriteGui(True)
        else:
            logging.warning("unknown argument " + argline[1] + ", ignored")
            bootstrap.AllPluginBootstrap().boot()
            app = ZouriteGui(False)
    else:   
        AllPluginBootstrap().boot()             
        app = ZouriteGui()
    # the hui is up and visible, so let's play
    app.run()


def configure(zcore, aPlugin):
    data = aPlugin.configure()
    while len(data) > 0:
    # print current value
        keys = data.keys()
        if plugin.PLUGIN_CONF_INFO in keys:
            print data[plugin.PLUGIN_CONF_INFO]
        
        if plugin.PLUGIN_CONF_WEBBROWSER in keys:
            url = data[plugin.PLUGIN_CONF_WEBBROWSER]
            print "please visit " + url
            webbrowser.open_new_tab(url)
        
        for key in keys:
        # handle special parameter
            if not key.startswith("__"):
                # standar parameter
                if data[key] is not None:
                    print "config param " + str(data[key])
                else:
                    input = raw_input("Please enter " + key + " :")
                    data[key] = input
        data = aPlugin.configure(data)
    print "configuration finished"
    print ""
    zcore.register_plugin(aPlugin)

    

    
def runSetup(force=False):
    '''
    This method should be run only if no plugin have been registered yet.
    This method try to load each plugin and mark them as registered e.g.
    to be load when zourite is instanciated.
    '''
    print "Zourite " + version.getInstance().getVersion()
    print "--------------------"
    print "Interactive Plugin Configuration Boostrap"
    print ""
    
    AllPluginBootstrap().boot() # inject all available plugin
    zcore = zourite.Zourite(auto_load=False) # create an instance of the core
    zcore.applyDefaultSettings()    
    for aPlugin in zourite.ZOURITE_AVAILABLE_PLUGIN: # for each available plugin
        print ">>>",aPlugin.get_plugin_id(),"<<<"    
        try:
            zcore.register_plugin(aPlugin) # register the plugin           
            if force:
                print "plugin configured, forced configuration..."
                configure(zcore, aPlugin)
            else:
                print "already configured !"
                print ""
        except plugin.ConfigurationRequiredExeption:
            print "not configured, starting configuration..."            
            configure(zcore, aPlugin)

    print "saving configuration..."
    zcore.save_registered_plugin() # save the current configuration of registered plugins
    print "configuration successful !!!"
        
if __name__ == "__main__":
        
    
    
    # check arg line if mock mode is required    
    argline = sys.argv
    if len(argline) > 1 and argline[1] == "setup":
        runSetup()
    elif len(argline) > 2 and argline[1] == "setup" and argline[2] == "force":
        runSetup(force=True)
    else:
        runGui(argline)

    


# -*- encoding: UTF-8 -*-

import plugin
import logging
import pickle
import os
import os.path

import dbus

from ..common import version # must be relative cause in the current package there is a module named zourite yet


version.getInstance().submitRevision("$Revision: 152 $")

ZOURITE_AVAILABLE_PLUGIN = None

NOT_YET_IMPLEMENTED = 'not yet implemented'

MEMORY_CACHE_STORE = "MEMORY_CACHE_STORE"
DISK_CACHE_STORE = "DISK_CACHE_STORE"

AVAILABLE_CACHE_STORE= [MEMORY_CACHE_STORE, DISK_CACHE_STORE]

def get_zourite_storage_dir(self):
    '''
    Compute the application storage dir.
    This is an utility function to retrieve the directory where zourite can
    store any file like settings or cached data.
    '''
    storage = os.path.expanduser("~")
    storage = os.path.join(storage, ".zourite")
    return storage

class Zourite(plugin.ZouritePlugin):
    '''
    The main class of the application (except the GUI). Zourite is a professional network
    application. The core concept is in plugin module and Zourite itself is a plugin which
    is intended for aggregate all other plugin. Currently there is only one plugin in other
    word one supported network: LinkedIn.
    '''
    registered_plugin = []
       
    settings = None
    
    
    def __init__(self, auto_load=True):
        # read registered plugin
        if auto_load:
            self.load_settings()
            self.apply_settings()
        
    
    def load_settings(self):
        '''
        load the saved settings
        '''
        self.ensure_zourite_conf_store()
        storage = self.get_zourite_settings_file()
        try:           
            file = open(storage,'rb')
            self.settings = pickle.load(file)
            file.close()
        except IOError, EOFError:
            logging.warning("failed to load the settings")
            self.settings  = Settings()
            
    
    def save_settings(self):
        '''
        save the current settings
        '''
        self.ensure_zourite_conf_store()
        storage = self.get_zourite_settings_file()
        try:           
            file = open(storage,'wb')
            pickle.dump(self.settings,file)
            file.close()
        except IOError:
            logging.warning("failed to save the settings")

    
    def apply_settings(self):
        self.load_registered_plugin()
   

    def get_zourite_settings_file(self):
        storage = get_zourite_storage_dir(self)
        storage = os.path.join(storage, "settings.pickle")
        return storage

   
    def load_registered_plugin(self):
        '''
        read the settings and load registered plugin
        '''        
        for pluginid in self.settings.registered_plugin_ids:
            
            plugin = self.get_available_plugin_by_id(pluginid)
            # which cache storage we will used ?
            if self.settings.cache_storage == MEMORY_CACHE_STORE:
                cachePlugin = MemoryCachePlugin(plugin)
            elif self.settings.cache_storage == DISK_CACHE_STORE:
                cachePlugin = DiskCachePlugin(plugin)
            else:
                cachePlugin = MemoryCachePlugin(plugin)
                logging.warning("the current cache store " + str(self.settings.cache_storage) + " is unknown")
                logging.warning("will use " + MEMORY_CACHE_STORE + " for " + plugin.get_plugin_id())
                
            if  self.settings.cache_strategy == CachePlugin.CACHE_PREFERED_STATEGY:
                cachePlugin.setCachePrefered()
            elif self.settings.cache_strategy == CachePlugin.NETWORK_DATA_PREFERED_STRATEGY:
                cachePlugin.setNetworkPrefered()
            else:
                logging.warning("the current cache strategy " + str(self.settings.cache_strategy) + " is unknow")
                logging.warning("will use " + cachePlugin.cacheStrategy + " for " + cachePlugin.get_plugin_id())
            
            self.register_plugin(cachePlugin)
       
    

        
      
    def save_registered_plugin(self):
        '''
        save current registered plugin in the settiongs then save the settings
        '''
        self.settings.registered_plugin_ids[:]
        for plugin in self.registered_plugin:
            self.settings.registered_plugin_ids.append(plugin.get_plugin_id())
        self.save_settings()

    def unregister_all_plugin(self):
        '''
        remove all registered plugin. This means all plugin are removed from memory
        but the settings is not modified.
        '''
        del self.registered_plugin[:]
        
    def register_plugin(self, plugin):
        '''
        Launch the plugin and remember it as registered.
        '''
        # this is a bare implementation 
        plugin.run_plugin()
        self.registered_plugin.append(plugin)
        
        
   
    def ensure_zourite_conf_store(self):
        storage = get_zourite_storage_dir(self)
        if os.path.exists(storage):
            pass
        else:
            os.makedirs(storage)
    
    
    def get_available_plugin_by_id(self, pluginid):
        for p in ZOURITE_AVAILABLE_PLUGIN:
            if p.get_plugin_id() == pluginid:
                return p
        mess = "the plugin for id " + str(pluginid) + " not found"
        raise RuntimeError(mess)
    
    def get_plugin_by_id(self, networkid):
        '''
        Look inside registered plugin list e.g. loaded plugin
        and return the which one with the given id
        '''
        for p in self.registered_plugin:
            if p.get_plugin_id() == networkid:
                return p
        mess = "the plugin for id " + str(networkid) + " not found"
        raise RuntimeError(mess)
    
    
    
    def get_short_profile(self, id, networkid):
        plugin = self.get_plugin_by_id(networkid)
        return plugin.getShortProfile(id)
    
    
    
    '''
    Facade 
    '''
    
    def resetCacheData(self):
        for plugin in self.registered_plugin:
            plugin.resetCacheData()
    
    def setForceRefresh(self, enabled):
       for plugin in self.registered_plugin:
           plugin.setBypassCache(enabled)
    
    def applyAndSaveSettings(self, new_settings):
        '''
        apply the new settings in arguments and save it
        '''
        self.unregister_all_plugin()
        self.settings = new_settings
        self.apply_settings()
        self.save_settings()
        
    def applyDefaultSettings(self):
        self.applyAndSaveSettings(Settings())
    
    def getRegisteredPlugin(self):
        return self.registered_plugin
    
    def getCurrentCacheStrategy(self):
        return self.settings.cache_strategy
    
    def getCurrentCacheStorage(self):
        return self.settings.cache_storage
    
    def getAvailableCacheStore(self):
        '''
        The availbale cache store.
        Each one can be uesed to modify the place zourite will cache data.
        For instance cache data may be stored in memory for efficience or
        on disk for persistence.
        '''
        return AVAILABLE_CACHE_STORE
    
    def getAvailableCacheStrategies(self):
        '''
        The available cache strategy. 
        Each one can be used to modify the way zourite use the cache.
        For instance it can prefer fresh data from the network and use information
        from cache only when the network can't be reached.
        '''
        return CachePlugin.AVAILABLE_CACHE_STRATEGIES
    
    def getAvailablePlugins(self):
        '''
        The available plugin.
        Each one can be activated and used by zourite as network to fetch information
        from. 
        '''
        return ZOURITE_AVAILABLE_PLUGIN
    
    def getPluginAmongAvailable(self,plugin_id):
        '''
        Search for a plugin among all available plugin.
        '''    
        availablePLugins = self.getAvailablePlugins()
        result = filter(lambda(x): x.get_plugin_id() == plugin_id, availablePLugins)
        plug = None
        if len(result) >  0:
            plug =  result[0]
        return plug
            
    
    def getShortProfilesForNetUpdateConnection(self, netUpdateConn):
        '''
        given a network update connection, return a tuple containin short profile
        for the existing connection and the new connection
        '''
        myconn = self.get_short_profile(netUpdateConn.connection.id, netUpdateConn.networkId)
        newconn = self.get_short_profile(netUpdateConn.newConnection.id, netUpdateConn.networkId)
        return myconn, newconn
    
    def getShortProfileForNetUpdateSelfConnection(self, netUpdateSelfConn):
        '''
        Given a netwokr update self connection, return the short profile for the new connection
        '''
        id = netUpdateSelfConn.newConnection.id
        networkid = netUpdateSelfConn.networkId
        return self.get_short_profile(id, networkid)
    
    def getGtkImageForProfile(self, shortProfile):
        '''
        return a gtk.Image for the profile
        '''
        plugin = self.get_plugin_by_id(shortProfile.networkId)
        return plugin.getGtkImage(shortProfile)
    
    def getGtkPixBufForProfile(self, shortProfile):
        '''
        return a gtk.Image for the profile
        '''
        plugin = self.get_plugin_by_id(shortProfile.networkId)
        return plugin.getGdkPixbuf(shortProfile)
    
    def getGtkPixBufForNetwork(self, networkId):
        '''
        return a pixbuf as picture for the network plugin
        '''
        plugin = self.get_plugin_by_id(networkId)
        return plugin.getPluginLogoPixbuf()
    
    def getGtkPixBufForPerson(self, person):
        '''
        return a pixbuf as picture for the network plugin
        '''    
        sp = self.get_short_profile(person.id, person.networkId)
        plugin = self.get_plugin_by_id(person.networkId)
        return plugin.getGdkPixbuf(sp)
    
      
    def getFullProfileForShortProfile(self, shortProfile):
        networkId = shortProfile.networkId
        contactId = shortProfile.id
        return self.get_plugin_by_id(networkId).getFullProfile(contactId) 
    
    def getFullProfileForShortPerson(self, person):
        networkId = person.networkId
        contactId = person.id
        return self.get_plugin_by_id(networkId).getFullProfile(contactId) 
    
    
    def sendConnectionRequestToProfile(self, shortProfile, subject, body):
        '''
        Send an invitation request to a given profile. Subject and body of the
        message must be provided
        '''
        plugin = self.get_plugin_by_id(shortProfile.networkId)
        plugin.sendInvitationRequest(shortProfile.id, subject, body)
    
    def sendMessageToProfile(self, shortProfile, subject, body):
        '''
        Send a message to a connected person.
        '''
        plugin = self.get_plugin_by_id(shortProfile.networkId)
        plugin.sendMessageRequest(shortProfile.id, subject, body)
    
    def getAllMyStatus(self):
        '''
        Return a set of all status grabbed from every network plugin.
        '''
        status = []
        for aPlugin in self.registered_plugin:
            aStatus = aPlugin.getMyStatus()
            if aStatus == None:
                pass
            elif len(aStatus) > 0:
                status.append(aStatus)
        return set(status)
        
    '''
    Interface for any network that can be used by zourite
    '''
    
    
    
    def getMyConnections(self):
        connections = []
        for aPlugin in self.registered_plugin:
            connections.extend(aPlugin.getMyConnections())
        return connections
    
    def setMyStatus(self, status):
        for aPlugin in self.registered_plugin:
            aPlugin.setMyStatus(status)
        
        # TODO : issue 17    
        # dbus-send --type=method_call --print-reply --dest=org.freedesktop.Telepathy.MissionControl /org/freedesktop/Telepathy/MissionControl org.freedesktop.Telepathy.MissionControl.SetPresence uint32:2 string:"I'm here"
        
    
    def clearMyStatus(self):
        for aPlugin in self.registered_plugin:
            aPlugin.clearMyStatus()
    

    def getNetworkUpdate(self):
        news = []
        for aPlugin in self.registered_plugin:
            news.extend(aPlugin.getNetworkUpdate())
        return news
    
    def sendMessageToNetwork(self, body):
        for aPlugin in self.registered_plugin:
            aPlugin.sendMessageToNetwork(body)


class Settings():
    '''
    Represents the settings for Zourite
    '''
   
    def __init__(self):
        self.registered_plugin_ids = []
        self.cache_strategy = CachePlugin.NETWORK_DATA_PREFERED_STRATEGY    
        self.cache_storage = MEMORY_CACHE_STORE
    

class CacheStore():
    '''
    Represent the underlying storage for a cache.
    '''
    
    def getData(self, key):
        '''
        Retrieve the data stored under the given key
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def setData(self, key, value):
        '''
        Store the given data under the given key.
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def removeKey(self, key):
        '''
        remove the data stored under the given key
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def containsKey(self, key):
        '''
        Test if this cache contains data for the given key
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def clear(self):
        '''
        Clear all data in this cache
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
     
class MemoryStore(CacheStore):
    '''
    In memory store for caching feature
    '''

    dataStore = None

    def __init__(self):
        self.dataStore = {}

    def clear(self):
        self.dataStore.clear()

    def removeKey(self, key):
        del self.dataStore[key]
        
    def getData(self, key):
        return self.dataStore[key]

    def setData(self, key, value):
        self.dataStore[key] = value

    def containsKey(self, key):
        return key in self.dataStore
        

class DiskStore(CacheStore):
    '''
    On disk (persistent) store for caching feature.    
    '''
    dataStore = None # path where data will be stored
    entity  = None
    
    def __init__(self, pluginid,  entity):
        self.entity = entity
        self.pluginid = pluginid        
        self._init_cache_dir()
    
    def __str__(self):
        return "Disk store for entity [" + self.entity + "]"
    
    def _init_cache_dir(self):
        self.dataStore = get_zourite_storage_dir(self)
        self.dataStore = os.path.join(self.dataStore,"cache")
        self.dataStore = os.path.join(self.dataStore,self.pluginid)
        self.dataStore = os.path.join(self.dataStore,self.entity)
        if os.path.exists(self.dataStore):
            pass
        else:
            os.makedirs(self.dataStore)
    
    def _get_storage_for_key(self, key):
        key_storage = os.path.join(self.dataStore, str(key))
        return key_storage

        
    def clear(self):
        for file in os.listdir(self.dataStore):
            filepath = os.path.join(self.dataStore, file)
            os.remove(filepath)
        os.removedirs(self.dataStore)
        self._init_cache_dir()

    def removeKey(self, key):
        key_storage = self._get_storage_for_key(key)
        os.remove(key_storage)
        
    def getData(self, key):
        key_storage = self._get_storage_for_key(key)
        try:
            file = open(key_storage,'rb')
            data = pickle.load(file)
            file.close()
            
        except IOError:
            logging.error("failed to read data " + str(key) + " from cache " + self.__str__())           
            raise
        
        return data

    def setData(self, key, value):
        key_storage = self._get_storage_for_key(key)
        try:
            file = open(key_storage,'wb')
            pickle.dump(value, file)
            file.close()
            
        except IOError:
            logging.warn("failed to write data " + str(key) + " into cache " + self.__str__())
                

    def containsKey(self, key):
        key_storage = self._get_storage_for_key(key)
        return os.path.exists(key_storage)


class CachePlugin(plugin.ZouritePlugin):
    '''
    This class is used by zourite to enable caching on regular plugin.
    The main goal is to provide a not connected usability of Zourite by
    using data previously loaded by the cache.
    '''
    
    shortProfileCache = None
    fullProfileCache = None
    connectionCache = None
    networkUpdateCache = None
    
    DUMMY_KEY = 1   # used for a call that don't have a key
    
    CACHE_PREFERED_STATEGY = "CACHE_PREFERED_STATEGY"
    NETWORK_DATA_PREFERED_STRATEGY = "NETWORK_DATA_PREFERED_STRATEGY"    
        
    AVAILABLE_CACHE_STRATEGIES = [CACHE_PREFERED_STATEGY, NETWORK_DATA_PREFERED_STRATEGY]
    
    cacheStrategy = None
    
    forceRefresh = False # bypass the strategy and force call to the legacy plugin
    

    def __init__(self, realPlugin):
        self.pluginImpl = realPlugin
        self.setNetworkPrefered()
        self._forceRefresh = False
    
    def _getMethod(self, fonction, key, cache):
        '''
        Appelle la fonction en paramètre en prenant en compte le cache.        
        '''
        if self.cacheStrategy == self.CACHE_PREFERED_STATEGY and not self.forceRefresh :
            if cache.containsKey(key) :
                value = cache.getData(key)
            else:
                value = fonction()
                cache.setData(key,value)
            return value
        elif self.cacheStrategy == self.NETWORK_DATA_PREFERED_STRATEGY or self.forceRefresh:
            try:
                value = fonction()
                cache.setData(key,value)
            except plugin.UnvailableException:
                if cache.containsKey(key):
                    value = cache.getData(key)
                else:
                    raise
            return value
        else:
            return fonction()

    def setBypassCache(self, enable):
        self.forceRefresh = enable
        
    def resetCacheData(self):
        '''
        Delete data in cache
        '''
        self.connectionCache.clear()
        self.fullProfileCache.clear()
        self.networkUpdateCache.clear()
        self.shortProfileCache.clear()
        
        
    def setCachePrefered(self):
        self.cacheStrategy = self.CACHE_PREFERED_STATEGY
    
    def setNetworkPrefered(self):
        self.cacheStrategy = self.NETWORK_DATA_PREFERED_STRATEGY
        
    def run_plugin(self):
        self.pluginImpl.run_plugin()
    
    def get_plugin_id(self):
        return self.pluginImpl.get_plugin_id()
    
    def getMyConnections(self):
        # the key used have no importance because the current method have no parameter so no key is required for the cache       
        return self._getMethod(lambda: self.pluginImpl.getMyConnections(), self.DUMMY_KEY, self.connectionCache)
            
    def getMyStatus(self):
        return self.pluginImpl.getMyStatus()
        
    def setMyStatus(self, status):
        self.pluginImpl.setMyStatus(status)
          
    def clearMyStatus(self):
        self.pluginImpl.clearMyStatus()

    def getNetworkUpdate(self):
        return self._getMethod(lambda: self.pluginImpl.getNetworkUpdate(), self.DUMMY_KEY, self.networkUpdateCache)
        
    def getShortProfile(self,contactId):
        return self._getMethod(lambda: self.pluginImpl.getShortProfile(contactId), contactId, self.shortProfileCache)
    
    def getFullProfile(self, contactId):
        return self._getMethod(lambda: self.pluginImpl.getFullProfile(contactId), contactId, self.fullProfileCache)
    
    def getMyShortProfile(self):
        return self.pluginImpl.getMyShortProfile()
    
    def getMyFulllProfile(self):
        return self.pluginImpl.getMyFullProfile()
    
    def getGtkImage(self, shortProfile):
        return self.pluginImpl.getGtkImage(shortProfile)
    
    def getGdkPixbuf(self, shortProfile):
        return self.pluginImpl.getGdkPixbuf(shortProfile)
    
    
    def getPluginLogoPixbuf(self):
        return self.pluginImpl.getPluginLogoPixbuf()

    def sendInvitationRequest(self, contactId, subject, body):
        self.pluginImpl.sendInvitationRequest(contactId,subject,body)
    
    def sendMessageRequest(self, contactId, subject, body):
        self.pluginImpl.sendMessageRequest(contactId,subject, body)
        
    def sendMessageToNetwork(self, body):
        self.pluginImpl.sendMessageToNetwork(body)
        
        
class MemoryCachePlugin(CachePlugin):
    '''
    Cache plugin that store data in memory
    '''
    def __init__(self, realPlugin):
        CachePlugin.__init__(self, realPlugin)
        self.shortProfileCache = MemoryStore()
        self.fullProfileCache = MemoryStore()
        self.connectionCache = MemoryStore()
        self.networkUpdateCache = MemoryStore()
        
        
class DiskCachePlugin(CachePlugin):
    '''
    Cache plugin that store data on disk (e.g. in a persistent way)
    '''
    def __init__(self, realPlugin):
        CachePlugin.__init__(self, realPlugin)
        plugin_id = realPlugin.get_plugin_id()
        self.shortProfileCache = DiskStore(plugin_id,"short_profile")
        self.fullProfileCache = DiskStore(plugin_id,"full_profile")
        self.connectionCache= DiskStore(plugin_id,"connection")
        self.networkUpdateCache = DiskStore(plugin_id, "network_update")


# -*- encoding: UTF-8 -*-
'''
Created on 3 mars 2010

@author: thierry
'''

from zourite.core import plugin
import linkedin
import pickle
import logging
import gtk
import gtk.gdk
import datetime


from zourite.common import version


version.getInstance().submitRevision("$Revision: 151 $")

PLUGIN_NAME = "linkedin"

class LinkedInPlugin(plugin.ZouritePlugin):

     
    
    
    API_KEY = ''
    SECRET_KEY = ''
    li = None;
    access_token = None
    
    
    unavailable = False # the LinkedIn API has raised a general Exception

       
    
 
    
    def retrieve_access_token(self):
        # essaye de charger l'acces token
        try:
            access_token_file = open(plugin.get_plugin_file_location(PLUGIN_NAME,'li_access_token'),'rb');
            access_token = pickle.load(access_token_file)
        except IOError:
            # pas d'acess token on en génère un autre
            raise plugin.ConfigurationRequiredExeption()            

        return access_token;
    
    
    def configure(self, data=None):
        if data is None:
            data = {}
            plugin.ensure_plugin_config_store_exist(PLUGIN_NAME)
            token = self.li.getRequestToken(None);
            data['token'] = token
            url = self.li.getAuthorizeUrl(token);
            data[plugin.PLUGIN_CONF_INFO] = "visit the provided url to authorize Zourite to access your LinkedIn account." \
                                            " LinkedIn will give you a verifierNote the verifier number that Zourite will ask you for." 
            data[plugin.PLUGIN_CONF_WEBBROWSER] = url
            data['verifier'] = None
            data[plugin.PLUGIN_CONF_STEP] = "VERIFIER"
            return data
        
        elif data[plugin.PLUGIN_CONF_STEP] == 'VERIFIER':
            verifier = int(data["verifier"])
            token = data['token']
            access_token = self.li.getAccessToken(token, verifier);
            try:
                access_token_file = open(plugin.get_plugin_file_location(PLUGIN_NAME,'li_access_token'),'wb');
                pickle.dump(access_token,access_token_file)
                access_token_file.close()
            except IOError:
                logging.warning("the access token could not be saved")
            return {}
        else :
            raise plugin.ConfigurationRequiredExeption() # may raise a specific 




    def _tcr_method(self, method, *arg):
        '''
        call a method and catch any LinkedInDomException then translate it
        into the appropriate ZouriteException
        '''
        if ( self.unavailable ) :
            # try to reinit the connection            
            self.li.reinitConnection()   
            self.unavailable = False         
            
        try:
        
            return method(*arg)
        
        except linkedin.LinkedInDomException, domExc:
            # parse the dom 
            statusDom = domExc.errorDom[0].getElementsByTagName("status")
            if len(statusDom) > 0 :
                status = statusDom[0].firstChild.nodeValue
                message = domExc.getDomMessage()
                if status == "401" and message.find("Expired access token") > -1:                    
                    logging.warn("LinkedIn API raised an exception: configuration is required")
                    raise plugin.ConfigurationRequiredExeption()
                elif status == "403" and message.find("Throttle limit") >  -1:
                    logging.warn("LinkedIn API raised an exception: throttle limit reached")
                    raise plugin.UnvailableException()
                else:
                    logging.error("LinkedIn API raised an exception but it contain an unknow status " + status + " or message")
                    raise 
            else:
                logging.error("LinkedIn API raised an exception that " + PLUGIN_NAME + " can't handle because there is no status")
                raise
            
        except linkedin.LinkedInException:
            logging.warning(PLUGIN_NAME + " catch a general exception, mark the service as unvailable")
            self.unavailable = True
            raise plugin.UnvailableException()
  
    #---------------------------------------------------------------------
    #
    # implementation de l'interface ZouritePlugin
    #
    #---------------------------------------------------------------------

    
    def getMyShortProfile(self):
        profileDom = self._tcr_method(lambda:self.li.profile_api.getMyShortProfile(self.access_token))
        profile = self.ShortProfile(profileDom)
        return profile
    
    def getShortProfile(self, contactId):
        profileDom = self._tcr_method(lambda: self.li.profile_api.getShortProfile(contactId, self.access_token))
        
        
        profile = ShortProfile(profileDom)
        return profile
    
    def getFullProfile(self, contactId):        
        profileDom = self._tcr_method(lambda:self.li.profile_api.getFullProfile(contactId, self.access_token))                
        profile = FullProfile(profileDom)
        return profile
    
    def getMyFullProfile(self):
        profileDom = self._tcr_method(lambda:self.li.profile_api.getMyFullProfile(self.access_token))
        profile = FullProfile(profileDom)
        return profile
    
    def getGtkImage(self, shortProfile):
        '''
        return a gtk.Image for the profil
        '''
        
        return shortProfile.picture.getGtkImage() # return a pixbuf it's perhaps better
    
    def getGdkPixbuf(self, shortProfile):
        return shortProfile.picture.getGdkPixbuf()
    
    
    def getPluginLogoPixbuf(self):
        return gtk.gdk.pixbuf_new_from_file("linkedin.png")
        
    def get_plugin_id(self):
        return PLUGIN_NAME
    
    def run_plugin(self):
        api_key_file = open(plugin.get_plugin_file_location(PLUGIN_NAME,'li_api_key'));
        try:
            self.API_KEY = api_key_file.readline();
        finally:
            api_key_file.close();
        secret_key_file = open(plugin.get_plugin_file_location(PLUGIN_NAME,'li_secret_key'));
        try:
            self.SECRET_KEY = secret_key_file.readline();
        finally:
            secret_key_file.close();

        self.li = linkedin.LinkedIn(self.API_KEY, self.SECRET_KEY);
        self.access_token = self.retrieve_access_token()
    
    def getMyConnections(self):
        people = self._tcr_method(self._getMyConnections)
           
        return people

    def getMyStatus(self):
        return self._tcr_method(lambda:self.li.status_api.getMyStatus(self.access_token));

    def setMyStatus(self, status):
        self._tcr_method(lambda:self.li.status_api.setMyStatus(status, self.access_token));
  
    def clearMyStatus(self):
        self._tcr_method(lambda: self.li.status_api.clearMyStatus(self.access_token));
    
    
    def sendInvitationRequest(self, contactId, subject, body):
        # retrieve an auth token: this is specific to linkedin API
        auth_token = ""
        api_std_profile_dom = self.li.profile_api.getApiStandardProfile(contactId, self.access_token)
        hdDomLst = api_std_profile_dom.getElementsByTagName('http-header')
        if len(hdDomLst) > 0:
            name = hdDomLst[0].getElementsByTagName('name')[0].firstChild.nodeValue
            if name == "x-li-auth-token" :
                auth_token = hdDomLst[0].getElementsByTagName('value')[0].firstChild.nodeValue        
        if auth_token == "":
            pass # TODO raise a plugin exception             
        self.li.communication_api.sendInvitation(contactId, subject, body, auth_token, self.access_token)
        
    
    def sendMessageRequest(self, contactId, subject, body):
        # TODO make some HTML escape in subject and body ?
        self._tcr_method(lambda:self.li.communication_api.sendMessage([contactId],subject, body,self.access_token))
        
    def sendMessageToNetwork(self, body):
        self._tcr_method(lambda:self.li.network_api.sendNewsMessage(body, self.access_token))

    def getNetworkUpdate(self):
        network_update = self._tcr_method(self._getNetworkUpdate)
            
        return network_update;

  
    class NetworkUpdateFactory():
        def createNetworkUpdate(self, update):
            '''
            instanciate a NetworkUpdate or subclass according to the provided
            dom update. Return None when the factory canot handle the entwork update
            '''
            networkUpdate = None
            type = update.getElementsByTagName("update-type")[0].firstChild.nodeValue
            if type == 'CONN':
                networkUpdate = NetworkUpdateConnection(update)
            elif type == 'NCON':
                networkUpdate = NetworkUpdateSelfConnection(update)
            elif type == 'STAT':
                networkUpdate = NetworkUpdateStatus(update)
            else:
                logging.warning("the connection type " + type + " is not yet supported")
            return networkUpdate

    def __str__(self):
        return "zourite plugin for the LinkedIn social network"

    def _getNetworkUpdate(self):
        updateDom = self.li.network_api.getNetworkUpdate(self.access_token)
        network_update = []
        nfactory = self.NetworkUpdateFactory()
        for update in updateDom:
            networkUpdate = nfactory.createNetworkUpdate(update)
            if networkUpdate is not None :
                network_update.append(networkUpdate)
        
        return network_update

    def _getMyConnections(self):
        peopleDom = self.li.connections_api.getMyConnections(self.access_token)
        people = []
        for p in peopleDom:
            person = ShortProfile(p)
            people.append(person)
        
        return people


    
class ShortProfile(plugin.ShortProfile):
     '''
     ShortProfile implementation by this plugin
     '''
     def __init__(self, pdom):
         plugin.ShortProfile.__init__(self)
         id = pdom.getElementsByTagName('id')[0].firstChild.nodeValue
         fn = pdom.getElementsByTagName('first-name')[0].firstChild.nodeValue
         ln = pdom.getElementsByTagName('last-name')[0].firstChild.nodeValue
         headline = pdom.getElementsByTagName('headline')[0].firstChild.nodeValue
         pictDomLst = pdom.getElementsByTagName('picture-url')
         # for some people the API don't return the picture even if there is one...
         if len(pictDomLst) > 0: 
             pictureUrl =  pictDomLst[0].firstChild.nodeValue
             self.picture = plugin.ImageProxyUrl(id,PLUGIN_NAME,pictureUrl)
         else:
             self.picture = plugin.ImageProxyNone()
             
         self.id = id
         self.firstname = fn
         self.lastname = ln
         self.headline = headline 
         
      
         
         self.networkId = PLUGIN_NAME

class FullProfile(ShortProfile,plugin.FullProfile):
     '''
     FullProfile implementation by this plugin
     '''
     def __init__(self, pdom):
         plugin.FullProfile.__init__(self)
         ShortProfile.__init__(self, pdom)
         # init full profile attributes
         locdom = pdom.getElementsByTagName('location')
         if len(locdom) > 0:
             self.location = locdom[0].getElementsByTagName('name')[0].firstChild.nodeValue
         inddom =pdom.getElementsByTagName('industry')
         if len(inddom) > 0:
             self.industry = inddom[0].firstChild.nodeValue
         sumdom = pdom.getElementsByTagName('summary')
         if len(sumdom) > 0:
             if sumdom[0].firstChild != None :
                 self.summary = sumdom[0].firstChild.nodeValue
         positions = pdom.getElementsByTagName('positions')
         if len (positions) > 0:
             posdom = positions[0].getElementsByTagName('position')
             for aPos in posdom:
                 position = Position(aPos)
                 self.positions.append(position
                                       )
class Position(plugin.Position):
     '''
     Position implementation by linkedin plugin
     '''
     
     def __init__(self, pdom):
         title = pdom.getElementsByTagName('title')[0].firstChild.nodeValue
         compdom = pdom.getElementsByTagName('company')[0]
         company = compdom.getElementsByTagName('name')[0].firstChild.nodeValue
         startDateLst = pdom.getElementsByTagName('start-date')
         if len(startDateLst) > 0 :
             startDom = startDateLst[0]
             year = startDom.getElementsByTagName('year')[0].firstChild.nodeValue
             month = "1"
             monthDom = startDom.getElementsByTagName('month')
             if len(monthDom) > 0 :
                 month = monthDom[0].firstChild.nodeValue
              
             plugin.Position.__init__(self,title, company, datetime.date(int(year),int(month),1))
         else:
             plugin.Position.__init__(self,title, company)
         
         sumdom = pdom.getElementsByTagName('summary')
         if len(sumdom) > 0:
             aSumDom = sumdom[0]
             # warning this element of the dom can be empty
             if aSumDom.firstChild !=None :
                 self.summary = aSumDom.firstChild.nodeValue
       
         
         
class NetworkUpdateConnection(plugin.NetworkUpdateConnection):
     '''
     Network Update of type connection
     '''
     def __init__(self,updateDom):
         
     
         timestamp = updateDom.getElementsByTagName("timestamp")[0].firstChild.nodeValue
       
         updateContent = updateDom.getElementsByTagName("update-content")[0]
         p = updateContent.getElementsByTagName("person")[0];
     

         conn = Person(p);
         cxs = updateContent.getElementsByTagName("connections")[0];
         newp = cxs.getElementsByTagName("person")[0];
         newconn = Person(newp);
         
         # unlike Java super initialization can occure here, 'cause simply a call to a function
     
         plugin.NetworkUpdateConnection.__init__(self, timestamp, conn, newconn, PLUGIN_NAME)
        

class NetworkUpdateSelfConnection(plugin.NetworkUpdateSelfConnection):
     '''
     Network Update of type connection
     '''
     def __init__(self,updateDom):
         
     
         timestamp = updateDom.getElementsByTagName("timestamp")[0].firstChild.nodeValue
       
         updateContent = updateDom.getElementsByTagName("update-content")[0]
         p = updateContent.getElementsByTagName("person")[0];    
         conn = Person(p);         
         
         # unlike Java super initialization can occure here, 'cause simply a call to a function     
         plugin.NetworkUpdateSelfConnection.__init__(self, timestamp, conn, PLUGIN_NAME)
     

class NetworkUpdateStatus(plugin.NetworkUpdateStatus):
    '''
    A contact have update his status    
    '''
    def __init__(self, updateDom):

         timestamp = updateDom.getElementsByTagName("timestamp")[0].firstChild.nodeValue
       
         updateContent = updateDom.getElementsByTagName("update-content")[0]
         p = updateContent.getElementsByTagName("person")[0];    
         connection = Person(p);         
         
         status = None
         statusDom = p.getElementsByTagName("current-status")         
         if len (statusDom) > 0 :
            status = statusDom[0].firstChild.nodeValue
         
         plugin.NetworkUpdateStatus.__init__(self, timestamp, connection, status, PLUGIN_NAME)
class Person(plugin.Person):
     '''
     linkedin implementation of a zourite person
     '''

     def __init__(self, pdom):
         '''
         intanciate a Person or subclass according to the provided
         dom person
         '''
         plugin.Person.__init__(self)
    
         fn = pdom.getElementsByTagName('first-name')[0].firstChild.nodeValue
         ln = pdom.getElementsByTagName('last-name')[0].firstChild.nodeValue
         id = pdom.getElementsByTagName('id')[0].firstChild.nodeValue
         headline = pdom.getElementsByTagName('headline')[0].firstChild.nodeValue
            
         self.id = id
         self.firstname = fn
         self.lastname = ln
         self.headline = headline     
         self.networkId = PLUGIN_NAME
        



if __name__ == "__main__":
    LinkedInPlugin().run_plugin()





# -*- encoding: UTF-8 -*-
'''
Created on 3 mars 2010

@author: thierry
'''

import  logging
import  os.path
import  urllib
import  datetime
import  gtk
import  gtk.gdk

from ..common import version # must be relative cause in the current package there is a module named zourite yet


version.getInstance().submitRevision("$Revision: 151 $")

NOT_YET_IMPLEMENTED = 'not yet implemented'

PLUGIN_CONF_INFO = "__info__"
PLUGIN_CONF_WEBBROWSER = "__webbrowser__" 
PLUGIN_CONF_STEP = "__step__"

class ZouritePlugin():
    '''
    Interface for any network that can be used by zourite
    '''
    def run_plugin(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def configure(self, data=None):
        '''
        configure the plugin.
        The configuration may be a multiple step process and the data parameter
        contains user provided information. Call this method with no parameter
        to initiate a configuration process. The method return a set of required
        parameter you can pass to a subsequent call as data parameter. And so on.
        When the method return an empty set it means that the configuration process
        is finished.
        
        '''
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def get_plugin_id(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)

    def getMyConnections(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)

    def getMyStatus(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)

    def setMyStatus(self, status):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def clearMyStatus(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)

    def getNetworkUpdate(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getShortProfile(self,contactId):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getFullProfile(self, contactId):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getMyShortProfile(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getMyFulllProfile(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getGtkImage(self, shortProfile):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getGdkPixbuf(self, shortProfile):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def getPluginLogoPixbuf(self):
        raise RuntimeError(NOT_YET_IMPLEMENTED)

    def sendInvitationRequest(self, contactId, subject, body):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def sendMessageRequest(self, contactId, subject, body):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
    
    def sendMessageToNetwork(self, body):
        raise RuntimeError(NOT_YET_IMPLEMENTED)
        
class NetworkUpdate():
    '''
    base class of any network update
    '''
    
    networkId = ""
   
    timestamp = None # when the event has occured
    

    def __init__(self, timestamp, networkId):
        self.timestamp = timestamp        
        self.networkId = networkId
        

class NetworkUpdateConnection(NetworkUpdate):
    '''
    Network Update of type connection. This is used  when a contact have connected
    to another people
    '''
    connection = None
    newConnection = None
    def __init__(self, timestamp, connection, newConnection, networkId):
        NetworkUpdate.__init__(self, timestamp, networkId)
        self.connection = connection
        self.newConnection = newConnection
    
        
    def __str__(self):
        return "%s %s is now connected to %s" % (self.connection.firstname, self.connection.lastname, str(self.newConnection))

class NetworkUpdateStatus(NetworkUpdate):
    '''
    A connection have update its status
    '''
    connection = None
    status = ""
    
    def __init__(self, timestamp, connection, status, networkId):
        NetworkUpdate.__init__(self, timestamp, networkId)
        self.status = status
        
    def __str__(self):
        return "%s %s says %s" % (self.connection.firstname, self.connection.lastname, self.status) 

class NetworkUpdateSelfConnection(NetworkUpdateConnection):
    '''
    The user have made a new connection
    '''
    def __init__(self, timestamp, newConn, networkId):
        NetworkUpdateConnection.__init__(self, timestamp, None, newConn, networkId)

    def __str__(self):
        return "You are now connected to %s" % (str(self.newConnection))        

class Person():
    '''
    Some few data for a connection. This is only some information like firstname, headline and so.
    For more advanced information use Profile instance. 
    '''
    id = ""  # only relevant for a given networkId
    networkId = ""
    firstname = ""
    lastname = ""
    headline = ""

    def __init__(self):
        self.id = ""
        self.firstname = ""
        self.lastname = ""
        self.headline = ""
        self.networkId = ""
         
    def __str__(self): 
        return "%s %s as %s" % (self.firstname, self.lastname, self.headline)


class ShortProfile(Person):
    '''
    a profile with basic information
    '''    
    picture = None
    email = None
    phone = None
    
    def __init__(self):
        Person.__init__(self)
        self.picture = None
        self.email = None
        self.phone = None


class ImageProxy():    
    '''
    This class can be used to store a  picture attribute for ShortProfile.
    When the plugin is asked for the picture it can retrieve it by this object 
    instead of calling the network API.
    '''
    
    def getGtkImage(self):
        raise RuntimeError("not yet implemented")
    
    def getGdkPixbuf(self):
        raise RuntimeError("not yet implemented")

class ImageProxyFile(ImageProxy):
    '''
    Implementation of ImageProxy that retrieve a picture from file
    These data are stored in the plugin home directory for further use
    '''
    
    
    
    def __init__(self, pluginName, filename):
        self.pluginName = pluginName
        self.filename = filename
        

    def getGtkImage(self):
        # check if image is on plugin cache
        file = get_plugin_file_location(self.pluginName, self.filename)
        image = gtk.Image()
        image.set_from_file(file)
        return image;
    
    def getGdkPixbuf(self):
        # check if image is on plugin cache
        file = get_plugin_file_location(self.pluginName, self.filename)
        pixbuf = gtk.gdk.pixbuf_new_from_file(file)        
        return pixbuf

        
        

class ImageProxyNone(ImageProxy):
    '''
    This can be used when no image is available
    '''
    
    def getGtkImage(self):
        image = gtk.Image();
        image.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
        return image
    
    def getGdkPixbuf(self):
        return gtk.gdk.pixbuf_new_from_file("general_default_avatar.png")
    
class ImageProxyUrl(ImageProxy):
    '''
    Implementation of an ImageProxy that retrieve the picture from an url. This 
    is currently used by LinkedIn plugin.
    '''
    
    url= None
    filename = None
    
    def __init__(self, contactId, pluginName, url):
        '''
        plugin : the plugin whom location will be used to store a local cache for
        the picture
        url: the web url of the picture
        '''
        self.url = url
        self.filename = contactId
        self.pluginName = pluginName
    
    
    def getGtkImage(self):
        # check if image is on plugin cache
        file = get_plugin_file_location(self.pluginName, self.filename)
        if os.path.exists(file):
            pass;
        else:
            # retrieve the file from the web
            urllib.urlretrieve(self.url,file)
        image = gtk.Image()
        image.set_from_file(file)
        return image;
    
    def getGdkPixbuf(self):
        # check if image is on plugin cache
        file = get_plugin_file_location(self.pluginName, self.filename)
        if os.path.exists(file):
            pass;
        else:
            # retrieve the file from the web
            urllib.urlretrieve(self.url,file)
        pixbuf = gtk.gdk.pixbuf_new_from_file(file)        
        return pixbuf
    
class FullProfile(ShortProfile):
    '''
    a profile with all available information
    '''
    summary = ""
    industry = ""
    location = ""
    positions = []
    
    def __init__(self):
        ShortProfile.__init__(self)
        self.summary = ""
        self.industry = ""
        self.location = ""
        self.positions = []

class Position():
    title = ""
    summary = ""
    company = ""
    dateDebut = None
    
    def __init__(self,title, company, dateDebut=None):    
        self.title = title
        self.company = company
        self.dateDebut = dateDebut
        self.company = ""
    
def get_plugin_file_location(pluginame, filename):
    '''
    return the full path name of a file for a given plugin
    '''
    result =  os.path.expanduser("~")
    result =  os.path.join(result,".zourite")
    result =  os.path.join(result,pluginame)
    result =  os.path.join(result,filename)
    
    return result 
  
def ensure_plugin_config_store_exist(pluginame):
    '''
    Vérifie que la zone de stockage de la configuration du plugin exist
    et la crée le cas échéant
    '''
    storage =  os.path.expanduser("~")
    storage =  os.path.join(storage,".zourite")
    storage =  os.path.join(storage,pluginame)
    if os.path.exists(storage):
        pass
    else:
        os.makedirs(storage)

class ZouriteException(Exception):
    pass
            
class UnvailableException(ZouriteException):
    '''
    This exception my be thrown by any plugin method that rely on
    a network (I/O) resource or a bandwith limit (like LinkedIn) when
    the method can't be achieve right now. The handler of this exception 
    is suggested to use any cached data.
    '''
class ConfigurationRequiredExeption(ZouriteException):
    '''
    Thrown by the plugin when it miss correct configuration to run.
    This exception must be caught by zourite either in a command line interface 
    or g GUI to bring up a wizard to help configuring the plugin.
    '''



"""
LinkedIn OAuth Api Access 
Version: 0.3
License: MIT
Author: Max Lynch <max@mendotasoft.com>
Website: http://mendotasoft.com, http://maxlynch.com
Date Released: 11/23/2009

Enjoy!
"""

import hashlib
import httplib
import time
import urllib2
from xml.dom.minidom import parseString
import logging
import socket


import oauth as oauth

from zourite.common import version


version.getInstance().submitRevision("$Revision: 145 $")

class LinkedIn():
    LI_SERVER = "api.linkedin.com"
    LI_API_URL = "https://api.linkedin.com"

    REQUEST_TOKEN_URL = LI_API_URL + "/uas/oauth/requestToken"
    AUTHORIZE_URL = LI_API_URL + "/uas/oauth/authorize"
    ACCESS_TOKEN_URL = LI_API_URL + "/uas/oauth/accessToken"

    status_api = None
    connection_api = None
    network_api = None
    profile_api = None
    communication_api = None


    def __init__(self, api_key, secret_key):
        self.api_key = api_key
        self.secret_key = secret_key
        self.initConnection()
        
        
    def initConnection(self):
       

        self.connection = httplib.HTTPSConnection(self.LI_SERVER)
        self.consumer = oauth.OAuthConsumer(self.api_key, self.secret_key)
        self.sig_method = oauth.OAuthSignatureMethod_HMAC_SHA1()

        self.status_api = StatusApi(self)
        self.connections_api = ConnectionsApi(self)
        self.network_api = NetworkApi(self)
        self.profile_api = ProfileApi(self)
        self.communication_api = CommunicationApi(self)
        
    def reinitConnection(self):
        self.connection.close()
        self.initConnection()

    def getRequestToken(self, callback):
        """
        Get a request token from linkedin
        """
        oauth_consumer_key = self.api_key

        oauth_request = oauth.OAuthRequest.from_consumer_and_token(self.consumer,
            callback=callback,
            http_url=self.REQUEST_TOKEN_URL)
        oauth_request.sign_request(self.sig_method, self.consumer, None)


        self.connection.request(oauth_request.http_method,
            self.REQUEST_TOKEN_URL, headers=oauth_request.to_header())
        response = self.connection.getresponse().read()

        token = oauth.OAuthToken.from_string(response)
        return token

    def getAuthorizeUrl(self, token):
        """
        Get the URL that we can redirect the user to for authorization of our
        application.
        """
        oauth_request = oauth.OAuthRequest.from_token_and_callback(token=token, http_url=self.AUTHORIZE_URL)
        return oauth_request.to_url()

    def getAccessToken(self, token, verifier):
        """
        Using the verifier we obtained through the user's authorization of
        our application, get an access code.

        Note: token is the request token returned from the call to getRequestToken

        @return an OAuthToken object with the access token.  Use it like this:
            token.key -> Key
            token.secret -> Secret Key
        """
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, token=token, verifier=verifier, http_url=self.ACCESS_TOKEN_URL)
        oauth_request.sign_request(self.sig_method, self.consumer, token)

        self.connection.request(oauth_request.http_method, self.ACCESS_TOKEN_URL, headers=oauth_request.to_header())
        response = self.connection.getresponse()
        return oauth.OAuthToken.from_string(response.read())

    """
    More functionality coming soon...
    """

class LinkedInApi():
    def __init__(self, linkedin):
        self.linkedin = linkedin
        
    def _call_and_handle_low_level_error(self, method):
        try:
            method()
        except (socket.error, socket.herror, socket.gaierror, socket.timeout):            
            raise LinkedInException("LinkedIn API : socket connection failed")
        except (httplib.CannotSendRequest):
            raise LinkedInException("LinkedIn API : http communication failed")

    def doApiRequest(self, url, access_token):
        """
        make a call to linkedin and return the dom response.
        some check are done so exception are raised in case of error
        """
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(self.linkedin.consumer, token=access_token, http_url=url)
        oauth_request.sign_request(self.linkedin.sig_method, self.linkedin.consumer, access_token)
        self._call_and_handle_low_level_error(lambda:self.linkedin.connection.request(oauth_request.http_method, url, headers=oauth_request.to_header()))
        xml = self.linkedin.connection.getresponse().read()
        dom = parseString(xml)
        errorDom = dom.getElementsByTagName('error')
        if len(errorDom) != 0  :
            raise LinkedInDomException(errorDom)
        return dom;

    def doApiPutRequest(self, url, body, access_token):
        """
        make a PUT call to linkedin and return the http status        
        """
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(self.linkedin.consumer, token=access_token, http_method="PUT", http_url=url)
        oauth_request.sign_request(self.linkedin.sig_method, self.linkedin.consumer, access_token)
        self._call_and_handle_low_level_error(lambda:self.linkedin.connection.request(oauth_request.http_method, url, body=body, headers=oauth_request.to_header()))
        resp = self.linkedin.connection.getresponse()
        status = resp.status        
        resp.read() # this mandatory to keep the connection in a consistent state
        return status;
        
    def doApiPostRequest(self, url, body, access_token):
        """
        make a POST call to linkedin and return the http status        
        """
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(self.linkedin.consumer, token=access_token, http_method="POST", http_url=url)
        oauth_request.sign_request(self.linkedin.sig_method, self.linkedin.consumer, access_token)
        self._call_and_handle_low_level_error(lambda:self.linkedin.connection.request(oauth_request.http_method, url, body=body, headers=oauth_request.to_header()))
        resp = self.linkedin.connection.getresponse()
        status = resp.status        
        resp.read() # this mandatory to keep the connection in a consistent state
        return status;
    
    def doApiDeleteRequest(self, url, access_token):
        """
        make a DELETE call to linkedin and return the http status        
        """
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(self.linkedin.consumer, token=access_token, http_method="DELETE", http_url=url)
        oauth_request.sign_request(self.linkedin.sig_method, self.linkedin.consumer, access_token)
        self._call_and_handle_low_level_error(lambda:self.linkedin.connection.request(oauth_request.http_method, url, headers=oauth_request.to_header()))
        resp = self.linkedin.connection.getresponse()
        status = resp.status        
        resp.read() # this mandatory to keep the connection in a consistent state
        return status;


class LinkedInException(Exception):
        
    def __init__(self, mess):
        self.message = mess

    def __str__(self):
        return self.message

class LinkedInDomException(LinkedInException):
    '''
    Wrappe any error returned by a call to LinkedIn API
    '''
    errorDom = None

    def __init__(self,errorDom):
        self.errorDom = errorDom
        LinkedInException.__init__(self, self.getDomMessage())
        

    def __str__(self):
        return self.getDomMessage()

    def getDomMessage(self):
        '''
        return the original message from the xml dom
        '''
        return  self.errorDom[0].getElementsByTagName('message')[0].firstChild.nodeValue



class CommunicationApi(LinkedInApi):
    
    POST_MESSAGE_URL = LinkedIn.LI_API_URL + "/v1/people/~/mailbox"
    
    def __init(self, linkedin):
        LinkedInApi.__init__(self, linkedin)
    
    def sendMessage(self, recipients, subject, body, access_token):
        '''
        send a message to every person in recipients list
        - recipients : list of person ID
        - subject, message : no HTML 
        '''
        xmlPost = u"<?xml version='1.0' encoding='UTF-8'?>" \
                + "<mailbox-item>" \
                    + "<recipients>"
        for recipient in recipients:
            xmlPost = xmlPost + "<recipient> <person path='/people/" + recipient + "' />" \
                            + "</recipient>" 
        xmlPost = xmlPost + "</recipients>" \
                + "<subject>" + subject + "</subject>" \
                + "<body>" + body + "</body>" \
                + "</mailbox-item>"
        status = self.doApiPostRequest(self.POST_MESSAGE_URL, xmlPost, access_token)
        if status == 201:
            pass
        else:
            raise LinkedInException("LinkedIn response : " + str(status))
    
    def sendInvitation(self, id, subject, body, x_li_auth_token, access_token):
        '''
        send an invitation to join the user network
        The x_li_auth_token must be a string following the pattern name:value
        '''
        auth_token = x_li_auth_token.split(":")
        auth_token_name = auth_token[0]
        auth_token_value = auth_token[1]
        xmlPost = u"<?xml version='1.0' encoding='UTF-8'?>" \
                + "<mailbox-item>" \
                    + "<recipients>" \
                        + "<recipient>" \
                            + " <person path='/people/d=" + id + "' />" \
                        + "</recipient>" \
                    + "</recipients>" \
                    + "<subject>" + subject + "</subject>" \
                    + "<body>" + body + "</body>" \
                    + "<item-content>" \
                        + "<invitation-request>" \
                            + "<connect-type>friend</connect-type>" \
                            + "<authorization>" \
                                + "<name>" + auth_token_name + "</name>" \
                                + "<value>" + auth_token_value + "</value>" \
                            + "</authorization>" \
                        + "</invitation-request>" \
                    + "</item-content>" \
                + "</mailbox-item>"
        status = self.doApiPostRequest(self.POST_MESSAGE_URL, xmlPost, access_token)
        if status == 201:
            pass
        else:
            raise LinkedInException("LinkedIn response : " + str(status))
        
class ProfileApi(LinkedInApi):
    
    GET_SELF_PROFILE_URL = LinkedIn.LI_API_URL + "/v1/people/~"
    GET_CONNECTION_PROFILE_URL = LinkedIn.LI_API_URL + "/v1/people/id="

    SHORT_PROFILE_ATTRIBUTES = ":(id,first-name,last-name,headline,picture-url)"
    FULL_PROFILE_ATTRIBUTES = ":(id,first-name,last-name,headline,picture-url,location,industry,summary,positions)"
    API_STANDARD_PROFILE_ATTRIBUTES = ":(api-standard-profile-request)"
    
    def __init(self, linkedin):
        LinkedInApi.__init__(self, linkedin)
    
    def getMyShortProfile(self, access_token):
        '''
        return the xml DOM for the person. The xml contains the profil.
        '''
        dom = self.doApiRequest(self.GET_SELF_PROFILE_URL + self.SHORT_PROFILE_ATTRIBUTES, access_token)
        personDom  = dom.getElementsByTagName('person')[0]
        return personDom

    def getShortProfile(self, contactId, access_token):
        '''
        return the xml DOM for a given contact idenified by its id.
        The xml contains the profil.
        '''
        dom = self.doApiRequest(self.GET_CONNECTION_PROFILE_URL + contactId                            
                                + self.SHORT_PROFILE_ATTRIBUTES, access_token)
        personDom  = dom.getElementsByTagName('person')[0]
        return personDom
        
    def getMyFullProfile(self, access_token):
        '''
        return the xml DOM for the person. The xml contains the profil.
        '''
        dom = self.doApiRequest(self.GET_SELF_PROFILE_URL + self.FULL_PROFILE_ATTRIBUTES, access_token)
        personDom  = dom.getElementsByTagName('person')[0]
        return personDom

    def getFullProfile(self, contactId, access_token):
        '''
        return the xml DOM for a given contact idenified by its id.
        The xml contains the profil.
        '''
        dom = self.doApiRequest(self.GET_CONNECTION_PROFILE_URL + contactId
                                + self.FULL_PROFILE_ATTRIBUTES, access_token)
        personDom  = dom.getElementsByTagName('person')[0]
        return personDom
    
    def getApiStandardProfile(self, contactId, access_token):
        '''
        return the dom containing the url for API standard profile
        '''
        dom = self.doApiRequest(self.GET_CONNECTION_PROFILE_URL + contactId
                                + self.API_STANDARD_PROFILE_ATTRIBUTES, access_token)
        api_std_profile  = dom.getElementsByTagName('api-standard-profile-request')[0]
        return api_std_profile
        
        

class StatusApi(LinkedInApi):
    
    GET_STATUS_SELF_URL = LinkedIn.LI_API_URL + "/v1/people/~:(current-status)"
    SET_STATUS_SELF_URL = LinkedIn.LI_API_URL + "/v1/people/~/current-status"
    
    def __init__(self, linkedin):
        LinkedInApi.__init__(self, linkedin)
        
    def getMyStatus(self, access_token):
        '''
        return the status as a string
        '''
        dom  = self.doApiRequest(self.GET_STATUS_SELF_URL, access_token)
        peopleDom = dom.getElementsByTagName('person')
        myself = peopleDom[0]
        statusDom = myself.getElementsByTagName('current-status')
        if len(statusDom) == 0:
            mystatus = None
        else:
            mystatus = statusDom[0].firstChild.nodeValue
        return mystatus;
    
    def setMyStatus(self, status, access_token):
        # construct the xml body request
        xmlStatus = '<?xml version="1.0" encoding="UTF-8"?>' + "<current-status>" + status + "</current-status>"
        status  = self.doApiPutRequest(self.SET_STATUS_SELF_URL, xmlStatus, access_token)
        if status != 204:
            raise LinkedInException("LinkedIn response : " + str(status))
    
    def clearMyStatus(self, access_token):
        '''
        clear the status of the user definied by the access_token
        '''
        status   = self.doApiDeleteRequest(self.SET_STATUS_SELF_URL, access_token)       
        if status != 204:
            raise LinkedInException("LinkedIn response : " + str(status))

class ConnectionsApi(LinkedInApi):
    """
    How to get all of a user's connections:

        Note: This should happen after the linkedin redirect.  verifier is passed
        by LinkedIn back to your redirect page

        li = LinkedIn(LINKEDIN_API_KEY, LINKEDIN_SECRET_KEY)

        tokenObj = oauth.OAuthToken(requestTokenKey, requestTokenSecret)
        access_token = li.getAccessToken(tokenObj, verifier)

        connections = li.connections_api.getMyConnections(access_token)

        for c in connections:
            # Access c.firstname, c.lastname, etc.
    """

    CONNECTIONS_SELF = LinkedIn.LI_API_URL + "/v1/people/~/connections"
   
    def __init__(self, linkedin):
        LinkedInApi.__init__(self, linkedin)
    def getMyConnections(self, access_token):
        dom = self.doApiRequest(self.CONNECTIONS_SELF, access_token)

        peopleDom = dom.getElementsByTagName('person')

        return peopleDom

class NetworkApi(LinkedInApi):

    NETWORK_UPDATE = LinkedIn.LI_API_URL + "/v1/people/~/network"
    POST_NEWS_URL = LinkedIn.LI_API_URL + "/v1/people/~/person-activites"
    
    def __init__(self, linkedin):
        LinkedInApi.__init__(self, linkedin)

    def getNetworkUpdate(self, access_token):
        '''
        Retrieve all network update
        '''
        dom = self.doApiRequest(self.NETWORK_UPDATE, access_token)
        updateDom = dom.getElementsByTagName('update')

        return updateDom
    
    def sendNewsMessage(self, body, access_token):
        '''
        send a message to every contact in the network
        '''
        xmlPost = u"<?xml version='1.0' encoding='UTF-8'?>" \
                + "<activity locale=\"en_US\">" \
                    + "<content-type>linkedin-html</content-type>" \
                    + "<body>" + body + "</body>" \
                + "</activity>"
        status = self.doApiPostRequest(self.POST_NEWS_URL, xmlPost, access_token)
        if status == 201:
            pass
        else:
            raise LinkedInException("LinkedIn response : " + str(status))



# -*- encoding: UTF-8 -*-
'''
Created on Mar 24, 2010

@author: maemo
'''

from zourite.core import plugin
import pickle
import logging
import gtk
import gtk.gdk
import datetime

import atom
import gdata.contacts
import gdata.contacts.service

from zourite.common import version


version.getInstance().submitRevision("$Revision: 159 $")

PLUGIN_NAME = "gmail"

class GmailPlugin(plugin.ZouritePlugin):
    '''
    The gmail plugin only provide contact on the address book of the gmail account.
    '''
    
    gmail_credential = None
   
    gd_client = None
    
    
    def _retrieve_gmail_credential(self):
        # essaye de charger l'acces token
        try:
            gmail_file = open(plugin.get_plugin_file_location(PLUGIN_NAME,'gmail_credential'),'rb');
            gmail_credential = pickle.load(gmail_file)
            gmail_file.close()
        except IOError:
            # pas d'acess token on en génère un autre
            raise plugin.ConfigurationRequiredExeption()            
    
        return gmail_credential;
    
    def _configure_gmail_account(self, user, password):
 
        gc = GmailCredential()
        gc.email = user
        gc.password = password
        try:
            gmail_file = open(plugin.get_plugin_file_location(PLUGIN_NAME,'gmail_credential'),'wb');
            pickle.dump(gc,gmail_file)
            gmail_file.close()
        except IOError:
            logging.warning("the access token could not be saved")
        
        return gc
    
    #---------------------------------------------------------------------
    #
    # implementation de l'interface ZouritePlugin
    #
    #---------------------------------------------------------------------
        
    def get_plugin_id(self):
        return PLUGIN_NAME
    
    def configure(self, data=None):
        '''
        launch the configuration wizard
        '''
        STEP_GMAIL_ACCOUNT = 'GMAIL_ACCOUNT'
        if data is None:
            data = {}
            plugin.ensure_plugin_config_store_exist(PLUGIN_NAME)
            # ask for user and password
            data['user'] = None
            data['password'] = None
            data[plugin.PLUGIN_CONF_INFO] = "Enter your gmail credentiel. The user must NOT include the @gmail.com part." 
            data[plugin.PLUGIN_CONF_STEP] = STEP_GMAIL_ACCOUNT
            return data
        elif data[plugin.PLUGIN_CONF_STEP] == STEP_GMAIL_ACCOUNT :
            user = data['user']
            password = data['password']
            self.gmail_credential = self._configure_gmail_account(user, password)
            return {}
      
    def getPluginLogoPixbuf(self):
        return gtk.gdk.pixbuf_new_from_file("gdata-contacts-icon.png")
    
    def run_plugin(self):
        self.gmail_credential = self._retrieve_gmail_credential()
        self.gd_client = gdata.contacts.service.ContactsService()
        self.gd_client.email = self.gmail_credential.email
        self.gd_client.password = self.gmail_credential.password
        self.gd_client.source = "bressure.net-zourite-1.0"
        try:
            self.gd_client.ProgrammaticLogin()
        except gdata.service.BadAuthentication:
            logging.exception("failed to log in gmail")
            raise plugin.ConfigurationRequiredExeption() 

    def getMyConnections(self):
        allContacts = self.gd_client.GetContactsFeed().entry
        people = []       
        for entry in allContacts:
            p = ShortProfile(entry, self.gd_client.GetPhoto(entry))
            people.append(p)
        
        return people

    def getShortProfile(self, contactId):
        allContacts = self.gd_client.GetContactsFeed().entry
        selected = filter(lambda(x):x.id.text.rpartition("/")[2] == contactId, allContacts)        
        profile = ShortProfile(selected[0], self.gd_client.GetPhoto(selected[0]))
        return profile
   
    def getGdkPixbuf(self, shortProfile):
        return shortProfile.picture.getGdkPixbuf()
    
    def getGtkImage(self, shortProfile):
        '''
        return a gtk.Image for the profil
        '''
        
        return shortProfile.picture.getGtkImage() # return a pixbuf it's perhaps better

    def getNetworkUpdate(self):
       return []

    def getFullProfile(self, contactId):
       raise plugin.UnvailableException()

    def getMyStatus(self):
        return None
        
    def setMyStatus(self, status):
        pass
    
    def clearMyStatus(self):
        pass





class GmailCredential():
    '''
    Simple objecy to store the gmail account credential
    '''
    email = ""
    password = ""

class Person(plugin.Person):
    
    def __init__(self, entry):
        plugin.Person.__init__(self)
        
        self.lastname= entry.title.text
               
        # TODO should choose the primary one
        organization = self._get_primary_organization(entry)
        if organization is not None :
            # Someyime the organization is an empty list
            if organization.org_title is not None:
                self.headline = organization.org_title.text
            if organization.org_name  is not None:
                self.headline = self.headline + " " + organization.org_name.text 
            
                                            
        self.networkId = PLUGIN_NAME
        self.id = entry.id.text.rpartition("/")[2]  # only the last part of this url will the id
        
    def _get_primary_organization(self, entry):
        '''
        Retrieve the primary organization or the first one 
        '''
        if isinstance(entry.organization,list):            
            if len(entry.organization) > 0:
                return entry.organization[0]
            else:
                return None
        elif isinstance(entry.organization, gdata.contacts.Organization):
            return entry.organization
        else:
            return None
    
        
class ShortProfile(Person, plugin.ShortProfile):
    
    def __init__(self, entry, hosted_image_binary):
        plugin.ShortProfile.__init__(self)
        Person.__init__(self, entry)
        
        if hosted_image_binary is None :
            self.picture = plugin.ImageProxyNone()
        else:
            filename = self.id
            file = plugin.get_plugin_file_location(PLUGIN_NAME, filename )
            image_file = open(file, 'wb')
            image_file.write(hosted_image_binary)
            image_file.close()
            self.picture = plugin.ImageProxyFile(PLUGIN_NAME, filename)

        # TODO should choose the work labeled one
        if len(entry.email) > 0 :
             self.email =  entry.email[0].address
        
        if len(entry.phone_number) > 0:
            self.phone = entry.phone_number[0].text
        



# -*- encoding: UTF-8 -*-
'''
Created on 30 mars 2010

@author: thierry
'''
from distutils.core import setup
setup(name='zourite',
      version='0.2.0-1',
      package_dir = {'': 'src'},
      packages=['zourite',
                'zourite.common',
                'zourite.core',
                'zourite.gui',
                'zourite.plugin',
                'zourite.plugin.gmail',
                'zourite.plugin.linkedin'],
      scripts=['scripts/zourite','scripts/zourite-setup','scripts/zourite-setup-force'],
      package_data={'zourite': ['*.png']},      
      author='Thierry Bressure',
      author_email='thierry@bressure.net',
      maintainer='Thierry Bressure',
      maintainer_email='zourite@bressure.net',
      url='http://blog.zourite.bressure.net',
      download_url='http://zourite.bressure.net',
      description='Zourite Professional Networking Application for N900',
      long_description='Zourite is a client for professional Network like LinkedIn. You can stay connected to your contacts and manage you network.',
      classifiers=[
          'Development Status :: 4 - Beta',
          "Environment :: Handhelds/PDA's",          
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',          
          'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Topic :: Communications :: Email',
          'Topic :: Office/Business',          
          ],

      )


